"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AutoPollConfigService = exports.POLL_EXPIRATION_TOLERANCE_MS = void 0;
const ConfigServiceBase_1 = require("./ConfigServiceBase.js");
const Utils_1 = require("./Utils.js");
exports.POLL_EXPIRATION_TOLERANCE_MS = 500;
class AutoPollConfigService extends ConfigServiceBase_1.ConfigServiceBase {
    constructor(configFetcher, options) {
        super(configFetcher, options);
        this.signalInitialization = () => { };
        this.stopToken = new Utils_1.AbortToken();
        this.pollIntervalMs = options.pollIntervalSeconds * 1000;
        this.pollExpirationMs = this.pollIntervalMs - exports.POLL_EXPIRATION_TOLERANCE_MS;
        const initialCacheSyncUp = this.syncUpWithCache();
        if (options.maxInitWaitTimeSeconds !== 0) {
            this.initialized = false;
            const initSignalPromise = new Promise(resolve => this.signalInitialization = resolve);
            this.initializationPromise = this.waitForInitializationAsync(initSignalPromise).then(success => {
                this.initialized = true;
                return success;
            });
        }
        else {
            this.initialized = true;
            this.initializationPromise = Promise.resolve(false);
        }
        this.readyPromise = this.getReadyPromise(this.initializationPromise, async (initializationPromise) => {
            await initializationPromise;
            return this.getCacheState(this.options.cache.getInMemory());
        });
        if (!options.offline) {
            this.startRefreshWorker(initialCacheSyncUp, this.stopToken);
        }
    }
    async waitForInitializationAsync(initSignalPromise) {
        if (this.options.maxInitWaitTimeSeconds < 0) {
            await initSignalPromise;
            return true;
        }
        const abortToken = new Utils_1.AbortToken();
        const success = await Promise.race([
            initSignalPromise.then(() => true),
            (0, Utils_1.delay)(this.options.maxInitWaitTimeSeconds * 1000, abortToken).then(() => false)
        ]);
        abortToken.abort();
        return success;
    }
    async getConfig() {
        this.options.logger.debug("AutoPollConfigService.getConfig() called.");
        function logSuccess(logger) {
            logger.debug("AutoPollConfigService.getConfig() - returning value from cache.");
        }
        let cachedConfig;
        if (!this.isOffline && !this.initialized) {
            cachedConfig = await this.options.cache.get(this.cacheKey);
            if (!cachedConfig.isExpired(this.pollIntervalMs)) {
                logSuccess(this.options.logger);
                return cachedConfig;
            }
            this.options.logger.debug("AutoPollConfigService.getConfig() - cache is empty or expired, waiting for initialization.");
            await this.initializationPromise;
        }
        cachedConfig = await this.options.cache.get(this.cacheKey);
        if (!cachedConfig.isExpired(this.pollIntervalMs)) {
            logSuccess(this.options.logger);
        }
        else {
            this.options.logger.debug("AutoPollConfigService.getConfig() - cache is empty or expired.");
        }
        return cachedConfig;
    }
    refreshConfigAsync() {
        this.options.logger.debug("AutoPollConfigService.refreshConfigAsync() called.");
        return super.refreshConfigAsync();
    }
    dispose() {
        this.options.logger.debug("AutoPollConfigService.dispose() called.");
        super.dispose();
        if (!this.stopToken.aborted) {
            this.stopRefreshWorker();
        }
    }
    onConfigFetched(newConfig) {
        this.signalInitialization();
        super.onConfigFetched(newConfig);
    }
    setOnlineCore() {
        this.startRefreshWorker(null, this.stopToken);
    }
    setOfflineCore() {
        this.stopRefreshWorker();
        this.stopToken = new Utils_1.AbortToken();
    }
    async startRefreshWorker(initialCacheSyncUp, stopToken) {
        this.options.logger.debug("AutoPollConfigService.startRefreshWorker() called.");
        let isFirstIteration = true;
        while (!stopToken.aborted) {
            try {
                const scheduledNextTimeMs = new Date().getTime() + this.pollIntervalMs;
                try {
                    await this.refreshWorkerLogic(isFirstIteration, initialCacheSyncUp);
                }
                catch (err) {
                    this.options.logger.autoPollConfigServiceErrorDuringPolling(err);
                }
                const realNextTimeMs = scheduledNextTimeMs - new Date().getTime();
                if (realNextTimeMs > 0) {
                    await (0, Utils_1.delay)(realNextTimeMs, stopToken);
                }
            }
            catch (err) {
                this.options.logger.autoPollConfigServiceErrorDuringPolling(err);
            }
            isFirstIteration = false;
            initialCacheSyncUp = null;
        }
    }
    stopRefreshWorker() {
        this.options.logger.debug("AutoPollConfigService.stopRefreshWorker() called.");
        this.stopToken.abort();
    }
    async refreshWorkerLogic(isFirstIteration, initialCacheSyncUp) {
        this.options.logger.debug("AutoPollConfigService.refreshWorkerLogic() - called.");
        const latestConfig = await (initialCacheSyncUp !== null && initialCacheSyncUp !== void 0 ? initialCacheSyncUp : this.options.cache.get(this.cacheKey));
        if (latestConfig.isExpired(this.pollExpirationMs)) {
            if (isFirstIteration ? !this.isOfflineExactly : !this.isOffline) {
                await this.refreshConfigCoreAsync(latestConfig);
            }
        }
        else if (isFirstIteration) {
            this.signalInitialization();
        }
    }
    getCacheState(cachedConfig) {
        if (cachedConfig.isEmpty) {
            return 0;
        }
        if (cachedConfig.isExpired(this.pollIntervalMs)) {
            return 2;
        }
        return 3;
    }
}
exports.AutoPollConfigService = AutoPollConfigService;
//# sourceMappingURL=AutoPollConfigService.js.map