"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LazyLoadOptions = exports.ManualPollOptions = exports.AutoPollOptions = exports.OptionsBase = exports.DataGovernance = exports.PollingMode = void 0;
const ConfigCatCache_1 = require("./ConfigCatCache.js");
const ConfigCatLogger_1 = require("./ConfigCatLogger.js");
const EventEmitter_1 = require("./EventEmitter.js");
const Hash_1 = require("./Hash.js");
const Hooks_1 = require("./Hooks.js");
const Polyfills_1 = require("./Polyfills.js");
const ProjectConfig_1 = require("./ProjectConfig.js");
var PollingMode;
(function (PollingMode) {
    PollingMode[PollingMode["AutoPoll"] = 0] = "AutoPoll";
    PollingMode[PollingMode["LazyLoad"] = 1] = "LazyLoad";
    PollingMode[PollingMode["ManualPoll"] = 2] = "ManualPoll";
})(PollingMode || (exports.PollingMode = PollingMode = {}));
var DataGovernance;
(function (DataGovernance) {
    DataGovernance[DataGovernance["Global"] = 0] = "Global";
    DataGovernance[DataGovernance["EuOnly"] = 1] = "EuOnly";
})(DataGovernance || (exports.DataGovernance = DataGovernance = {}));
class OptionsBase {
    constructor(sdkKey, clientVersion, options, defaultCacheFactory, eventEmitterFactory) {
        var _a, _b, _c;
        this.requestTimeoutMs = 30000;
        this.baseUrlOverriden = false;
        this.offline = false;
        if (!sdkKey) {
            throw new Error("Invalid 'sdkKey' value");
        }
        this.sdkKey = sdkKey;
        this.clientVersion = clientVersion;
        this.dataGovernance = (_a = options === null || options === void 0 ? void 0 : options.dataGovernance) !== null && _a !== void 0 ? _a : 0;
        switch (this.dataGovernance) {
            case 1:
                this.baseUrl = "https://cdn-eu.configcat.com";
                break;
            default:
                this.baseUrl = "https://cdn-global.configcat.com";
                break;
        }
        const eventEmitter = (_b = eventEmitterFactory === null || eventEmitterFactory === void 0 ? void 0 : eventEmitterFactory()) !== null && _b !== void 0 ? _b : new EventEmitter_1.NullEventEmitter();
        const hooks = new Hooks_1.Hooks(eventEmitter);
        const hooksWeakRef = new ((0, Polyfills_1.isWeakRefAvailable)() ? WeakRef : (0, Polyfills_1.getWeakRefStub)())(hooks);
        this.hooks = {
            hooks,
            hooksWeakRef,
            emit(eventName, ...args) {
                var _a, _b;
                return (_b = (_a = this.hooksWeakRef.deref()) === null || _a === void 0 ? void 0 : _a.emit(eventName, ...args)) !== null && _b !== void 0 ? _b : false;
            }
        };
        let logger;
        let cache;
        if (options) {
            logger = options.logger;
            cache = options.cache;
            if (options.requestTimeoutMs) {
                if (options.requestTimeoutMs < 0) {
                    throw new Error("Invalid 'requestTimeoutMs' value");
                }
                this.requestTimeoutMs = options.requestTimeoutMs;
            }
            if (options.baseUrl) {
                this.baseUrl = options.baseUrl;
                this.baseUrlOverriden = true;
            }
            if (options.flagOverrides) {
                this.flagOverrides = options.flagOverrides;
            }
            if (options.defaultUser) {
                this.defaultUser = options.defaultUser;
            }
            if (options.offline) {
                this.offline = options.offline;
            }
            (_c = options.setupHooks) === null || _c === void 0 ? void 0 : _c.call(options, hooks);
        }
        this.logger = new ConfigCatLogger_1.LoggerWrapper(logger !== null && logger !== void 0 ? logger : new ConfigCatLogger_1.ConfigCatConsoleLogger(), this.hooks);
        this.cache = cache
            ? new ConfigCatCache_1.ExternalConfigCache(cache, this.logger)
            : (defaultCacheFactory ? defaultCacheFactory(this) : new ConfigCatCache_1.InMemoryConfigCache());
    }
    yieldHooks() {
        const hooksWrapper = this.hooks;
        const hooks = hooksWrapper.hooks;
        delete hooksWrapper.hooks;
        return hooks !== null && hooks !== void 0 ? hooks : new Hooks_1.Hooks(new EventEmitter_1.NullEventEmitter());
    }
    getUrl() {
        return this.baseUrl + "/configuration-files/" + this.sdkKey + "/" + OptionsBase.configFileName + "?sdk=" + this.clientVersion;
    }
    getCacheKey() {
        return (0, Hash_1.sha1)(`${this.sdkKey}_${OptionsBase.configFileName}_${ProjectConfig_1.ProjectConfig.serializationFormatVersion}`);
    }
}
exports.OptionsBase = OptionsBase;
OptionsBase.configFileName = "config_v6.json";
class AutoPollOptions extends OptionsBase {
    constructor(sdkKey, sdkType, sdkVersion, options, defaultCacheFactory, eventEmitterFactory) {
        super(sdkKey, sdkType + "/a-" + sdkVersion, options, defaultCacheFactory, eventEmitterFactory);
        this.pollIntervalSeconds = 60;
        this.maxInitWaitTimeSeconds = 5;
        if (options) {
            if (options.pollIntervalSeconds != null) {
                this.pollIntervalSeconds = options.pollIntervalSeconds;
            }
            if (options.maxInitWaitTimeSeconds != null) {
                this.maxInitWaitTimeSeconds = options.maxInitWaitTimeSeconds;
            }
        }
        const maxSetTimeoutIntervalSecs = 2147483;
        if (!(typeof this.pollIntervalSeconds === "number" && 1 <= this.pollIntervalSeconds && this.pollIntervalSeconds <= maxSetTimeoutIntervalSecs)) {
            throw new Error("Invalid 'pollIntervalSeconds' value");
        }
        if (!(typeof this.maxInitWaitTimeSeconds === "number" && this.maxInitWaitTimeSeconds <= maxSetTimeoutIntervalSecs)) {
            throw new Error("Invalid 'maxInitWaitTimeSeconds' value");
        }
    }
}
exports.AutoPollOptions = AutoPollOptions;
class ManualPollOptions extends OptionsBase {
    constructor(sdkKey, sdkType, sdkVersion, options, defaultCacheFactory, eventEmitterFactory) {
        super(sdkKey, sdkType + "/m-" + sdkVersion, options, defaultCacheFactory, eventEmitterFactory);
    }
}
exports.ManualPollOptions = ManualPollOptions;
class LazyLoadOptions extends OptionsBase {
    constructor(sdkKey, sdkType, sdkVersion, options, defaultCacheFactory, eventEmitterFactory) {
        super(sdkKey, sdkType + "/l-" + sdkVersion, options, defaultCacheFactory, eventEmitterFactory);
        this.cacheTimeToLiveSeconds = 60;
        if (options) {
            if (options.cacheTimeToLiveSeconds != null) {
                this.cacheTimeToLiveSeconds = options.cacheTimeToLiveSeconds;
            }
        }
        if (!(typeof this.cacheTimeToLiveSeconds === "number" && 1 <= this.cacheTimeToLiveSeconds && this.cacheTimeToLiveSeconds <= 2147483647)) {
            throw new Error("Invalid 'cacheTimeToLiveSeconds' value");
        }
    }
}
exports.LazyLoadOptions = LazyLoadOptions;
//# sourceMappingURL=ConfigCatClientOptions.js.map