"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfigCatConsoleLogger = exports.LoggerWrapper = exports.FormattableLogMessage = exports.LogLevel = void 0;
exports.nameOfLogLevel = nameOfLogLevel;
const Utils_1 = require("./Utils.js");
var LogLevel;
(function (LogLevel) {
    LogLevel[LogLevel["Debug"] = 4] = "Debug";
    LogLevel[LogLevel["Info"] = 3] = "Info";
    LogLevel[LogLevel["Warn"] = 2] = "Warn";
    LogLevel[LogLevel["Error"] = 1] = "Error";
    LogLevel[LogLevel["Off"] = -1] = "Off";
})(LogLevel || (exports.LogLevel = LogLevel = {}));
function nameOfLogLevel(value) {
    return LogLevel[value];
}
class FormattableLogMessage {
    static from(...argNames) {
        return (strings, ...argValues) => new FormattableLogMessage(strings, argNames, argValues);
    }
    constructor(strings, argNames, argValues) {
        this.strings = strings;
        this.argNames = argNames;
        this.argValues = argValues;
    }
    get defaultFormattedMessage() {
        let cachedMessage = this.cachedDefaultFormattedMessage;
        if (cachedMessage === void 0) {
            cachedMessage = "";
            const { strings, argValues } = this;
            let i = 0;
            for (; i < strings.length - 1; i++) {
                cachedMessage += strings[i];
                cachedMessage += argValues[i];
            }
            cachedMessage += strings[i];
            this.cachedDefaultFormattedMessage = cachedMessage;
        }
        return cachedMessage;
    }
    toString() { return this.defaultFormattedMessage; }
}
exports.FormattableLogMessage = FormattableLogMessage;
class LoggerWrapper {
    get level() {
        var _a;
        return (_a = this.logger.level) !== null && _a !== void 0 ? _a : 2;
    }
    get eol() {
        var _a;
        return (_a = this.logger.eol) !== null && _a !== void 0 ? _a : "\n";
    }
    constructor(logger, hooks) {
        this.logger = logger;
        this.hooks = hooks;
    }
    isEnabled(logLevel) {
        return this.level >= logLevel;
    }
    log(level, eventId, message, exception) {
        var _a;
        if (this.isEnabled(level)) {
            this.logger.log(level, eventId, message, exception);
        }
        if (level === 1) {
            (_a = this.hooks) === null || _a === void 0 ? void 0 : _a.emit("clientError", message.toString(), exception);
        }
        return message;
    }
    debug(message) {
        this.log(4, 0, message);
    }
    configJsonIsNotPresent(defaultReturnValue) {
        return this.log(1, 1000, FormattableLogMessage.from("DEFAULT_RETURN_VALUE") `Config JSON is not present. Returning ${defaultReturnValue}.`);
    }
    configJsonIsNotPresentSingle(key, defaultParamName, defaultParamValue) {
        return this.log(1, 1000, FormattableLogMessage.from("KEY", "DEFAULT_PARAM_NAME", "DEFAULT_PARAM_VALUE") `Config JSON is not present when evaluating setting '${key}'. Returning the \`${defaultParamName}\` parameter that you specified in your application: '${defaultParamValue}'.`);
    }
    settingEvaluationFailedDueToMissingKey(key, defaultParamName, defaultParamValue, availableKeys) {
        return this.log(1, 1001, FormattableLogMessage.from("KEY", "DEFAULT_PARAM_NAME", "DEFAULT_PARAM_VALUE", "AVAILABLE_KEYS") `Failed to evaluate setting '${key}' (the key was not found in config JSON). Returning the \`${defaultParamName}\` parameter that you specified in your application: '${defaultParamValue}'. Available keys: [${availableKeys}].`);
    }
    settingEvaluationError(methodName, defaultReturnValue, ex) {
        return this.log(1, 1002, FormattableLogMessage.from("METHOD_NAME", "DEFAULT_RETURN_VALUE") `Error occurred in the \`${methodName}\` method. Returning ${defaultReturnValue}.`, ex);
    }
    settingEvaluationErrorSingle(methodName, key, defaultParamName, defaultParamValue, ex) {
        return this.log(1, 1002, FormattableLogMessage.from("METHOD_NAME", "KEY", "DEFAULT_PARAM_NAME", "DEFAULT_PARAM_VALUE") `Error occurred in the \`${methodName}\` method while evaluating setting '${key}'. Returning the \`${defaultParamName}\` parameter that you specified in your application: '${defaultParamValue}'.`, ex);
    }
    forceRefreshError(methodName, ex) {
        return this.log(1, 1003, FormattableLogMessage.from("METHOD_NAME") `Error occurred in the \`${methodName}\` method.`, ex);
    }
    fetchFailedDueToInvalidSdkKey() {
        return this.log(1, 1100, "Your SDK Key seems to be wrong. You can find the valid SDK Key at https://app.configcat.com/sdkkey");
    }
    fetchFailedDueToUnexpectedHttpResponse(statusCode, reasonPhrase) {
        return this.log(1, 1101, FormattableLogMessage.from("STATUS_CODE", "REASON_PHRASE") `Unexpected HTTP response was received while trying to fetch config JSON: ${statusCode} ${reasonPhrase}`);
    }
    fetchFailedDueToRequestTimeout(timeoutMs, ex) {
        return this.log(1, 1102, FormattableLogMessage.from("TIMEOUT") `Request timed out while trying to fetch config JSON. Timeout value: ${timeoutMs}ms`, ex);
    }
    fetchFailedDueToUnexpectedError(ex) {
        return this.log(1, 1103, "Unexpected error occurred while trying to fetch config JSON. It is most likely due to a local network issue. Please make sure your application can reach the ConfigCat CDN servers (or your proxy server) over HTTP.", ex);
    }
    fetchFailedDueToRedirectLoop() {
        return this.log(1, 1104, "Redirection loop encountered while trying to fetch config JSON. Please contact us at https://configcat.com/support/");
    }
    fetchReceived200WithInvalidBody(ex) {
        return this.log(1, 1105, "Fetching config JSON was successful but the HTTP response content was invalid.", ex);
    }
    fetchReceived304WhenLocalCacheIsEmpty(statusCode, reasonPhrase) {
        return this.log(1, 1106, FormattableLogMessage.from("STATUS_CODE", "REASON_PHRASE") `Unexpected HTTP response was received when no config JSON is cached locally: ${statusCode} ${reasonPhrase}`);
    }
    autoPollConfigServiceErrorDuringPolling(ex) {
        return this.log(1, 1200, "Error occurred during auto polling.", ex);
    }
    settingForVariationIdIsNotPresent(variationId) {
        return this.log(1, 2011, FormattableLogMessage.from("VARIATION_ID") `Could not find the setting for the specified variation ID: '${variationId}'.`);
    }
    configServiceCacheReadError(ex) {
        return this.log(1, 2200, "Error occurred while reading the cache.", ex);
    }
    configServiceCacheWriteError(ex) {
        return this.log(1, 2201, "Error occurred while writing the cache.", ex);
    }
    clientIsAlreadyCreated(sdkKey) {
        return this.log(2, 3000, FormattableLogMessage.from("SDK_KEY") `There is an existing client instance for the specified SDK Key. No new client instance will be created and the specified options are ignored. Returning the existing client instance. SDK Key: '${sdkKey}'.`);
    }
    userObjectIsMissing(key) {
        return this.log(2, 3001, FormattableLogMessage.from("KEY") `Cannot evaluate targeting rules and % options for setting '${key}' (User Object is missing). You should pass a User Object to the evaluation methods like \`getValueAsync()\` in order to make targeting work properly. Read more: https://configcat.com/docs/advanced/user-object/`);
    }
    dataGovernanceIsOutOfSync() {
        return this.log(2, 3002, "The `dataGovernance` parameter specified at the client initialization is not in sync with the preferences on the ConfigCat Dashboard. Read more: https://configcat.com/docs/advanced/data-governance/");
    }
    userObjectAttributeIsMissingPercentage(key, attributeName) {
        return this.log(2, 3003, FormattableLogMessage.from("KEY", "ATTRIBUTE_NAME", "ATTRIBUTE_NAME") `Cannot evaluate % options for setting '${key}' (the User.${attributeName} attribute is missing). You should set the User.${attributeName} attribute in order to make targeting work properly. Read more: https://configcat.com/docs/advanced/user-object/`);
    }
    userObjectAttributeIsMissingCondition(condition, key, attributeName) {
        return this.log(2, 3003, FormattableLogMessage.from("CONDITION", "KEY", "ATTRIBUTE_NAME", "ATTRIBUTE_NAME") `Cannot evaluate condition (${condition}) for setting '${key}' (the User.${attributeName} attribute is missing). You should set the User.${attributeName} attribute in order to make targeting work properly. Read more: https://configcat.com/docs/advanced/user-object/`);
    }
    userObjectAttributeIsInvalid(condition, key, reason, attributeName) {
        return this.log(2, 3004, FormattableLogMessage.from("CONDITION", "KEY", "REASON", "ATTRIBUTE_NAME") `Cannot evaluate condition (${condition}) for setting '${key}' (${reason}). Please check the User.${attributeName} attribute and make sure that its value corresponds to the comparison operator.`);
    }
    userObjectAttributeIsAutoConverted(condition, key, attributeName, attributeValue) {
        return this.log(2, 3005, FormattableLogMessage.from("CONDITION", "KEY", "ATTRIBUTE_NAME", "ATTRIBUTE_VALUE") `Evaluation of condition (${condition}) for setting '${key}' may not produce the expected result (the User.${attributeName} attribute is not a string value, thus it was automatically converted to the string value '${attributeValue}'). Please make sure that using a non-string value was intended.`);
    }
    configServiceCannotInitiateHttpCalls() {
        return this.log(2, 3200, "Client is in offline mode, it cannot initiate HTTP calls.");
    }
    configServiceMethodHasNoEffectDueToDisposedClient(methodName) {
        return this.log(2, 3201, FormattableLogMessage.from("METHOD_NAME") `The client object is already disposed, thus \`${methodName}()\` has no effect.`);
    }
    configServiceMethodHasNoEffectDueToOverrideBehavior(overrideBehavior, methodName) {
        return this.log(2, 3202, FormattableLogMessage.from("OVERRIDE_BEHAVIOR", "METHOD_NAME") `Client is configured to use the \`${overrideBehavior}\` override behavior, thus \`${methodName}()\` has no effect.`);
    }
    settingEvaluated(evaluateLog) {
        return this.log(3, 5000, FormattableLogMessage.from("EVALUATE_LOG") `${evaluateLog}`);
    }
    configServiceStatusChanged(status) {
        return this.log(3, 5200, FormattableLogMessage.from("MODE") `Switched to ${status.toUpperCase()} mode.`);
    }
}
exports.LoggerWrapper = LoggerWrapper;
class ConfigCatConsoleLogger {
    constructor(level = 2, eol = "\n") {
        this.level = level;
        this.eol = eol;
        this.SOURCE = "ConfigCat";
    }
    log(level, eventId, message, exception) {
        const [logMethod, levelString] = level === 4 ? [console.info, "DEBUG"] :
            level === 3 ? [console.info, "INFO"] :
                level === 2 ? [console.warn, "WARN"] :
                    level === 1 ? [console.error, "ERROR"] :
                        [console.log, nameOfLogLevel(level).toUpperCase()];
        const exceptionString = exception !== void 0 ? this.eol + (0, Utils_1.errorToString)(exception, true) : "";
        logMethod(`${this.SOURCE} - ${levelString} - [${eventId}] ${message}${exceptionString}`);
    }
}
exports.ConfigCatConsoleLogger = ConfigCatConsoleLogger;
//# sourceMappingURL=ConfigCatLogger.js.map