"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RolloutEvaluator = exports.EvaluateContext = void 0;
exports.evaluationDetailsFromDefaultValue = evaluationDetailsFromDefaultValue;
exports.evaluate = evaluate;
exports.evaluateAll = evaluateAll;
exports.checkSettingsAvailable = checkSettingsAvailable;
exports.isAllowedValue = isAllowedValue;
exports.handleInvalidReturnValue = handleInvalidReturnValue;
exports.getTimestampAsDate = getTimestampAsDate;
const EvaluateLogBuilder_1 = require("./EvaluateLogBuilder.js");
const Hash_1 = require("./Hash.js");
const ProjectConfig_1 = require("./ProjectConfig.js");
const Semver_1 = require("./Semver.js");
const User_1 = require("./User.js");
const Utils_1 = require("./Utils.js");
class EvaluateContext {
    get visitedFlags() { var _a; return (_a = this.$visitedFlags) !== null && _a !== void 0 ? _a : (this.$visitedFlags = []); }
    constructor(key, setting, user, settings) {
        this.key = key;
        this.setting = setting;
        this.user = user;
        this.settings = settings;
    }
    static forPrerequisiteFlag(key, setting, dependentFlagContext) {
        const context = new EvaluateContext(key, setting, dependentFlagContext.user, dependentFlagContext.settings);
        context.$visitedFlags = dependentFlagContext.visitedFlags;
        context.logBuilder = dependentFlagContext.logBuilder;
        return context;
    }
}
exports.EvaluateContext = EvaluateContext;
const targetingRuleIgnoredMessage = "The current targeting rule is ignored and the evaluation continues with the next rule.";
const missingUserObjectError = "cannot evaluate, User Object is missing";
const missingUserAttributeError = (attributeName) => `cannot evaluate, the User.${attributeName} attribute is missing`;
const invalidUserAttributeError = (attributeName, reason) => `cannot evaluate, the User.${attributeName} attribute is invalid (${reason})`;
class RolloutEvaluator {
    constructor(logger) {
        this.logger = logger;
    }
    evaluate(defaultValue, context) {
        this.logger.debug("RolloutEvaluator.evaluate() called.");
        let logBuilder = context.logBuilder;
        if (this.logger.isEnabled(3)) {
            context.logBuilder = logBuilder = new EvaluateLogBuilder_1.EvaluateLogBuilder(this.logger.eol);
            logBuilder.append(`Evaluating '${context.key}'`);
            if (context.user) {
                logBuilder.append(` for User '${JSON.stringify((0, User_1.getUserAttributes)(context.user))}'`);
            }
            logBuilder.increaseIndent();
        }
        let returnValue;
        try {
            let result, isValidReturnValue;
            if (defaultValue != null) {
                const settingType = context.setting.type;
                if (settingType >= 0 && !isCompatibleValue(defaultValue, settingType)) {
                    const settingTypeName = (0, ProjectConfig_1.nameOfSettingType)(settingType);
                    throw new TypeError("The type of a setting must match the type of the specified default value. "
                        + `Setting's type was ${settingTypeName} but the default value's type was ${typeof defaultValue}. `
                        + `Please use a default value which corresponds to the setting type ${settingTypeName}. `
                        + "Learn more: https://configcat.com/docs/sdk-reference/js/#setting-type-mapping");
                }
                result = this.evaluateSetting(context);
                returnValue = result.selectedValue.value;
                isValidReturnValue = typeof returnValue === typeof defaultValue;
            }
            else {
                result = this.evaluateSetting(context);
                returnValue = result.selectedValue.value;
                isValidReturnValue = isAllowedValue(returnValue);
            }
            if (!isValidReturnValue) {
                handleInvalidReturnValue(returnValue);
            }
            return result;
        }
        catch (err) {
            logBuilder === null || logBuilder === void 0 ? void 0 : logBuilder.resetIndent().increaseIndent();
            returnValue = defaultValue;
            throw err;
        }
        finally {
            if (logBuilder) {
                logBuilder.newLine(`Returning '${returnValue}'.`)
                    .decreaseIndent();
                this.logger.settingEvaluated(logBuilder.toString());
            }
        }
    }
    evaluateSetting(context) {
        let evaluateResult;
        const targetingRules = context.setting.targetingRules;
        if (targetingRules.length > 0 && (evaluateResult = this.evaluateTargetingRules(targetingRules, context))) {
            return evaluateResult;
        }
        const percentageOptions = context.setting.percentageOptions;
        if (percentageOptions.length > 0 && (evaluateResult = this.evaluatePercentageOptions(percentageOptions, void 0, context))) {
            return evaluateResult;
        }
        return { selectedValue: context.setting };
    }
    evaluateTargetingRules(targetingRules, context) {
        const logBuilder = context.logBuilder;
        logBuilder === null || logBuilder === void 0 ? void 0 : logBuilder.newLine("Evaluating targeting rules and applying the first match if any:");
        for (let i = 0; i < targetingRules.length; i++) {
            const targetingRule = targetingRules[i];
            const conditions = targetingRule.conditions;
            const isMatchOrError = this.evaluateConditions(conditions, targetingRule, context.key, context);
            if (isMatchOrError !== true) {
                if (isEvaluationError(isMatchOrError)) {
                    logBuilder === null || logBuilder === void 0 ? void 0 : logBuilder.increaseIndent().newLine(targetingRuleIgnoredMessage).decreaseIndent();
                }
                continue;
            }
            if (!(0, Utils_1.isArray)(targetingRule.then)) {
                return { selectedValue: targetingRule.then, matchedTargetingRule: targetingRule };
            }
            const percentageOptions = targetingRule.then;
            logBuilder === null || logBuilder === void 0 ? void 0 : logBuilder.increaseIndent();
            const evaluateResult = this.evaluatePercentageOptions(percentageOptions, targetingRule, context);
            if (evaluateResult) {
                logBuilder === null || logBuilder === void 0 ? void 0 : logBuilder.decreaseIndent();
                return evaluateResult;
            }
            logBuilder === null || logBuilder === void 0 ? void 0 : logBuilder.newLine(targetingRuleIgnoredMessage).decreaseIndent();
        }
    }
    evaluatePercentageOptions(percentageOptions, matchedTargetingRule, context) {
        const logBuilder = context.logBuilder;
        if (!context.user) {
            logBuilder === null || logBuilder === void 0 ? void 0 : logBuilder.newLine("Skipping % options because the User Object is missing.");
            if (!context.isMissingUserObjectLogged) {
                this.logger.userObjectIsMissing(context.key);
                context.isMissingUserObjectLogged = true;
            }
            return;
        }
        const percentageOptionsAttributeName = context.setting.percentageOptionsAttribute;
        const percentageOptionsAttributeValue = (0, User_1.getUserAttribute)(context.user, percentageOptionsAttributeName);
        if (percentageOptionsAttributeValue == null) {
            logBuilder === null || logBuilder === void 0 ? void 0 : logBuilder.newLine(`Skipping % options because the User.${percentageOptionsAttributeName} attribute is missing.`);
            if (!context.isMissingUserObjectAttributeLogged) {
                this.logger.userObjectAttributeIsMissingPercentage(context.key, percentageOptionsAttributeName);
                context.isMissingUserObjectAttributeLogged = true;
            }
            return;
        }
        logBuilder === null || logBuilder === void 0 ? void 0 : logBuilder.newLine(`Evaluating % options based on the User.${percentageOptionsAttributeName} attribute:`);
        const sha1Hash = (0, Hash_1.sha1)(context.key + userAttributeValueToString(percentageOptionsAttributeValue));
        const hashValue = parseInt(sha1Hash.substring(0, 7), 16) % 100;
        logBuilder === null || logBuilder === void 0 ? void 0 : logBuilder.newLine(`- Computing hash in the [0..99] range from User.${percentageOptionsAttributeName} => ${hashValue} (this value is sticky and consistent across all SDKs)`);
        let bucket = 0;
        for (let i = 0; i < percentageOptions.length; i++) {
            const percentageOption = percentageOptions[i];
            bucket += percentageOption.percentage;
            if (hashValue >= bucket) {
                continue;
            }
            logBuilder === null || logBuilder === void 0 ? void 0 : logBuilder.newLine(`- Hash value ${hashValue} selects % option ${i + 1} (${percentageOption.percentage}%), '${(0, EvaluateLogBuilder_1.valueToString)(percentageOption.value)}'.`);
            return { selectedValue: percentageOption, matchedTargetingRule, matchedPercentageOption: percentageOption };
        }
        throw new Error("Sum of percentage option percentages is less than 100.");
    }
    evaluateConditions(conditions, targetingRule, contextSalt, context) {
        let result = true;
        const logBuilder = context.logBuilder;
        let newLineBeforeThen = false;
        logBuilder === null || logBuilder === void 0 ? void 0 : logBuilder.newLine("- ");
        for (let i = 0; i < conditions.length; i++) {
            const condition = conditions[i];
            if (logBuilder) {
                if (!i) {
                    logBuilder.append("IF ")
                        .increaseIndent();
                }
                else {
                    logBuilder.increaseIndent()
                        .newLine("AND ");
                }
            }
            switch (condition.type) {
                case "UserCondition":
                    result = this.evaluateUserCondition(condition, contextSalt, context);
                    newLineBeforeThen = conditions.length > 1;
                    break;
                case "PrerequisiteFlagCondition":
                    result = this.evaluatePrerequisiteFlagCondition(condition, context);
                    newLineBeforeThen = true;
                    break;
                case "SegmentCondition":
                    result = this.evaluateSegmentCondition(condition, context);
                    newLineBeforeThen = !isEvaluationError(result) || result !== missingUserObjectError || conditions.length > 1;
                    break;
                default:
                    throw new Error();
            }
            const success = result === true;
            if (logBuilder) {
                if (!targetingRule || conditions.length > 1) {
                    logBuilder.appendConditionConsequence(success);
                }
                logBuilder.decreaseIndent();
            }
            if (!success) {
                break;
            }
        }
        if (targetingRule) {
            logBuilder === null || logBuilder === void 0 ? void 0 : logBuilder.appendTargetingRuleConsequence(targetingRule, result, newLineBeforeThen);
        }
        return result;
    }
    evaluateUserCondition(condition, contextSalt, context) {
        const logBuilder = context.logBuilder;
        logBuilder === null || logBuilder === void 0 ? void 0 : logBuilder.appendUserCondition(condition);
        if (!context.user) {
            if (!context.isMissingUserObjectLogged) {
                this.logger.userObjectIsMissing(context.key);
                context.isMissingUserObjectLogged = true;
            }
            return missingUserObjectError;
        }
        const userAttributeName = condition.comparisonAttribute;
        const userAttributeValue = (0, User_1.getUserAttribute)(context.user, userAttributeName);
        if (userAttributeValue == null || userAttributeValue === "") {
            this.logger.userObjectAttributeIsMissingCondition((0, EvaluateLogBuilder_1.formatUserCondition)(condition), context.key, userAttributeName);
            return missingUserAttributeError(userAttributeName);
        }
        let text, versionOrError, numberOrError, arrayOrError;
        switch (condition.comparator) {
            case 28:
            case 29:
                text = getUserAttributeValueAsText(userAttributeName, userAttributeValue, condition, context.key, this.logger);
                return this.evaluateTextEquals(text, condition.comparisonValue, condition.comparator === 29);
            case 20:
            case 21:
                text = getUserAttributeValueAsText(userAttributeName, userAttributeValue, condition, context.key, this.logger);
                return this.evaluateSensitiveTextEquals(text, condition.comparisonValue, context.setting.configJsonSalt, contextSalt, condition.comparator === 21);
            case 0:
            case 1:
                text = getUserAttributeValueAsText(userAttributeName, userAttributeValue, condition, context.key, this.logger);
                return this.evaluateTextIsOneOf(text, condition.comparisonValue, condition.comparator === 1);
            case 16:
            case 17:
                text = getUserAttributeValueAsText(userAttributeName, userAttributeValue, condition, context.key, this.logger);
                return this.evaluateSensitiveTextIsOneOf(text, condition.comparisonValue, context.setting.configJsonSalt, contextSalt, condition.comparator === 17);
            case 30:
            case 31:
                text = getUserAttributeValueAsText(userAttributeName, userAttributeValue, condition, context.key, this.logger);
                return this.evaluateTextSliceEqualsAnyOf(text, condition.comparisonValue, true, condition.comparator === 31);
            case 22:
            case 23:
                text = getUserAttributeValueAsText(userAttributeName, userAttributeValue, condition, context.key, this.logger);
                return this.evaluateSensitiveTextSliceEqualsAnyOf(text, condition.comparisonValue, context.setting.configJsonSalt, contextSalt, true, condition.comparator === 23);
            case 32:
            case 33:
                text = getUserAttributeValueAsText(userAttributeName, userAttributeValue, condition, context.key, this.logger);
                return this.evaluateTextSliceEqualsAnyOf(text, condition.comparisonValue, false, condition.comparator === 33);
            case 24:
            case 25:
                text = getUserAttributeValueAsText(userAttributeName, userAttributeValue, condition, context.key, this.logger);
                return this.evaluateSensitiveTextSliceEqualsAnyOf(text, condition.comparisonValue, context.setting.configJsonSalt, contextSalt, false, condition.comparator === 25);
            case 2:
            case 3:
                text = getUserAttributeValueAsText(userAttributeName, userAttributeValue, condition, context.key, this.logger);
                return this.evaluateTextContainsAnyOf(text, condition.comparisonValue, condition.comparator === 3);
            case 4:
            case 5:
                versionOrError = getUserAttributeValueAsSemVer(userAttributeName, userAttributeValue, condition, context.key, this.logger);
                return typeof versionOrError !== "string"
                    ? this.evaluateSemVerIsOneOf(versionOrError, condition.comparisonValue, condition.comparator === 5)
                    : versionOrError;
            case 6:
            case 7:
            case 8:
            case 9:
                versionOrError = getUserAttributeValueAsSemVer(userAttributeName, userAttributeValue, condition, context.key, this.logger);
                return typeof versionOrError !== "string"
                    ? this.evaluateSemVerRelation(versionOrError, condition.comparator, condition.comparisonValue)
                    : versionOrError;
            case 10:
            case 11:
            case 12:
            case 13:
            case 14:
            case 15:
                numberOrError = getUserAttributeValueAsNumber(userAttributeName, userAttributeValue, condition, context.key, this.logger);
                return typeof numberOrError !== "string"
                    ? this.evaluateNumberRelation(numberOrError, condition.comparator, condition.comparisonValue)
                    : numberOrError;
            case 18:
            case 19:
                numberOrError = getUserAttributeValueAsUnixTimeSeconds(userAttributeName, userAttributeValue, condition, context.key, this.logger);
                return typeof numberOrError !== "string"
                    ? this.evaluateDateTimeRelation(numberOrError, condition.comparisonValue, condition.comparator === 18)
                    : numberOrError;
            case 34:
            case 35:
                arrayOrError = getUserAttributeValueAsStringArray(userAttributeName, userAttributeValue, condition, context.key, this.logger);
                return typeof arrayOrError !== "string"
                    ? this.evaluateArrayContainsAnyOf(arrayOrError, condition.comparisonValue, condition.comparator === 35)
                    : arrayOrError;
            case 26:
            case 27:
                arrayOrError = getUserAttributeValueAsStringArray(userAttributeName, userAttributeValue, condition, context.key, this.logger);
                return typeof arrayOrError !== "string"
                    ? this.evaluateSensitiveArrayContainsAnyOf(arrayOrError, condition.comparisonValue, context.setting.configJsonSalt, contextSalt, condition.comparator === 27)
                    : arrayOrError;
            default:
                throw new Error();
        }
    }
    evaluateTextEquals(text, comparisonValue, negate) {
        return (text === comparisonValue) !== negate;
    }
    evaluateSensitiveTextEquals(text, comparisonValue, configJsonSalt, contextSalt, negate) {
        const hash = hashComparisonValue(text, configJsonSalt, contextSalt);
        return (hash === comparisonValue) !== negate;
    }
    evaluateTextIsOneOf(text, comparisonValues, negate) {
        const result = comparisonValues.indexOf(text) >= 0;
        return result !== negate;
    }
    evaluateSensitiveTextIsOneOf(text, comparisonValues, configJsonSalt, contextSalt, negate) {
        const hash = hashComparisonValue(text, configJsonSalt, contextSalt);
        const result = comparisonValues.indexOf(hash) >= 0;
        return result !== negate;
    }
    evaluateTextSliceEqualsAnyOf(text, comparisonValues, startsWith, negate) {
        for (let i = 0; i < comparisonValues.length; i++) {
            const item = comparisonValues[i];
            if (text.length < item.length) {
                continue;
            }
            const result = (startsWith ? text.lastIndexOf(item, 0) : text.indexOf(item, text.length - item.length)) >= 0;
            if (result) {
                return !negate;
            }
        }
        return negate;
    }
    evaluateSensitiveTextSliceEqualsAnyOf(text, comparisonValues, configJsonSalt, contextSalt, startsWith, negate) {
        const textUtf8 = (0, Utils_1.utf8Encode)(text);
        for (let i = 0; i < comparisonValues.length; i++) {
            const item = comparisonValues[i];
            const index = item.indexOf("_");
            const sliceLength = parseInt(item.slice(0, index));
            if (textUtf8.length < sliceLength) {
                continue;
            }
            const sliceUtf8 = startsWith ? textUtf8.slice(0, sliceLength) : textUtf8.slice(textUtf8.length - sliceLength);
            const hash = hashComparisonValueSlice(sliceUtf8, configJsonSalt, contextSalt);
            const result = hash === item.slice(index + 1);
            if (result) {
                return !negate;
            }
        }
        return negate;
    }
    evaluateTextContainsAnyOf(text, comparisonValues, negate) {
        for (let i = 0; i < comparisonValues.length; i++) {
            if (text.indexOf(comparisonValues[i]) >= 0) {
                return !negate;
            }
        }
        return negate;
    }
    evaluateSemVerIsOneOf(version, comparisonValues, negate) {
        let result = false;
        for (let i = 0; i < comparisonValues.length; i++) {
            const item = comparisonValues[i];
            if (!item.length) {
                continue;
            }
            const version2 = (0, Semver_1.parse)(item.trim());
            if (!version2) {
                return false;
            }
            if (!result && version.compare(version2) === 0) {
                result = true;
            }
        }
        return result !== negate;
    }
    evaluateSemVerRelation(version, comparator, comparisonValue) {
        const version2 = (0, Semver_1.parse)(comparisonValue.trim());
        if (!version2) {
            return false;
        }
        const comparisonResult = version.compare(version2);
        switch (comparator) {
            case 6: return comparisonResult < 0;
            case 7: return comparisonResult <= 0;
            case 8: return comparisonResult > 0;
            case 9: return comparisonResult >= 0;
        }
    }
    evaluateNumberRelation(number, comparator, comparisonValue) {
        switch (comparator) {
            case 10: return number === comparisonValue;
            case 11: return number !== comparisonValue;
            case 12: return number < comparisonValue;
            case 13: return number <= comparisonValue;
            case 14: return number > comparisonValue;
            case 15: return number >= comparisonValue;
        }
    }
    evaluateDateTimeRelation(number, comparisonValue, before) {
        return before ? number < comparisonValue : number > comparisonValue;
    }
    evaluateArrayContainsAnyOf(array, comparisonValues, negate) {
        for (let i = 0; i < array.length; i++) {
            const result = comparisonValues.indexOf(array[i]) >= 0;
            if (result) {
                return !negate;
            }
        }
        return negate;
    }
    evaluateSensitiveArrayContainsAnyOf(array, comparisonValues, configJsonSalt, contextSalt, negate) {
        for (let i = 0; i < array.length; i++) {
            const hash = hashComparisonValue(array[i], configJsonSalt, contextSalt);
            const result = comparisonValues.indexOf(hash) >= 0;
            if (result) {
                return !negate;
            }
        }
        return negate;
    }
    evaluatePrerequisiteFlagCondition(condition, context) {
        const logBuilder = context.logBuilder;
        logBuilder === null || logBuilder === void 0 ? void 0 : logBuilder.appendPrerequisiteFlagCondition(condition, context.settings);
        const prerequisiteFlagKey = condition.prerequisiteFlagKey;
        const prerequisiteFlag = context.settings[prerequisiteFlagKey];
        context.visitedFlags.push(context.key);
        if (context.visitedFlags.indexOf(prerequisiteFlagKey) >= 0) {
            context.visitedFlags.push(prerequisiteFlagKey);
            const dependencyCycle = (0, Utils_1.formatStringList)(context.visitedFlags, void 0, void 0, " -> ");
            throw new Error(`Circular dependency detected between the following depending flags: ${dependencyCycle}.`);
        }
        const prerequisiteFlagContext = EvaluateContext.forPrerequisiteFlag(prerequisiteFlagKey, prerequisiteFlag, context);
        logBuilder === null || logBuilder === void 0 ? void 0 : logBuilder.newLine("(").increaseIndent().newLine(`Evaluating prerequisite flag '${prerequisiteFlagKey}':`);
        const prerequisiteFlagEvaluateResult = this.evaluateSetting(prerequisiteFlagContext);
        context.visitedFlags.pop();
        const prerequisiteFlagValue = prerequisiteFlagEvaluateResult.selectedValue.value;
        if (typeof prerequisiteFlagValue !== typeof condition.comparisonValue) {
            if (isAllowedValue(prerequisiteFlagValue)) {
                throw new Error(`Type mismatch between comparison value '${condition.comparisonValue}' and prerequisite flag '${prerequisiteFlagKey}'.`);
            }
            else {
                handleInvalidReturnValue(prerequisiteFlagValue);
            }
        }
        let result;
        switch (condition.comparator) {
            case 0:
                result = prerequisiteFlagValue === condition.comparisonValue;
                break;
            case 1:
                result = prerequisiteFlagValue !== condition.comparisonValue;
                break;
            default:
                throw new Error();
        }
        logBuilder === null || logBuilder === void 0 ? void 0 : logBuilder.newLine(`Prerequisite flag evaluation result: '${(0, EvaluateLogBuilder_1.valueToString)(prerequisiteFlagValue)}'.`).newLine("Condition (").appendPrerequisiteFlagCondition(condition, context.settings).append(") evaluates to ").appendConditionResult(result).append(".").decreaseIndent().newLine(")");
        return result;
    }
    evaluateSegmentCondition(condition, context) {
        const logBuilder = context.logBuilder;
        logBuilder === null || logBuilder === void 0 ? void 0 : logBuilder.appendSegmentCondition(condition);
        if (!context.user) {
            if (!context.isMissingUserObjectLogged) {
                this.logger.userObjectIsMissing(context.key);
                context.isMissingUserObjectLogged = true;
            }
            return missingUserObjectError;
        }
        const segment = condition.segment;
        logBuilder === null || logBuilder === void 0 ? void 0 : logBuilder.newLine("(").increaseIndent().newLine(`Evaluating segment '${segment.name}':`);
        const segmentResult = this.evaluateConditions(segment.conditions, void 0, segment.name, context);
        let result = segmentResult;
        if (!isEvaluationError(result)) {
            switch (condition.comparator) {
                case 0:
                    break;
                case 1:
                    result = !result;
                    break;
                default:
                    throw new Error();
            }
        }
        if (logBuilder) {
            logBuilder.newLine("Segment evaluation result: ");
            (!isEvaluationError(result)
                ? logBuilder.append(`User ${(0, EvaluateLogBuilder_1.formatSegmentComparator)(segmentResult ? 0 : 1)}`)
                : logBuilder.append(result))
                .append(".");
            logBuilder.newLine("Condition (").appendSegmentCondition(condition).append(")");
            (!isEvaluationError(result)
                ? logBuilder.append(" evaluates to ").appendConditionResult(result)
                : logBuilder.append(" failed to evaluate"))
                .append(".");
            logBuilder
                .decreaseIndent()
                .newLine(")");
        }
        return result;
    }
}
exports.RolloutEvaluator = RolloutEvaluator;
function isEvaluationError(isMatchOrError) {
    return typeof isMatchOrError === "string";
}
function hashComparisonValue(value, configJsonSalt, contextSalt) {
    return hashComparisonValueSlice((0, Utils_1.utf8Encode)(value), configJsonSalt, contextSalt);
}
function hashComparisonValueSlice(sliceUtf8, configJsonSalt, contextSalt) {
    return (0, Hash_1.sha256)(sliceUtf8 + (0, Utils_1.utf8Encode)(configJsonSalt) + (0, Utils_1.utf8Encode)(contextSalt));
}
function userAttributeValueToString(userAttributeValue) {
    return typeof userAttributeValue === "string" ? userAttributeValue :
        userAttributeValue instanceof Date ? (userAttributeValue.getTime() / 1000) + "" :
            (0, Utils_1.isStringArray)(userAttributeValue) ? JSON.stringify(userAttributeValue) :
                userAttributeValue + "";
}
function getUserAttributeValueAsText(attributeName, attributeValue, condition, key, logger) {
    if (typeof attributeValue === "string") {
        return attributeValue;
    }
    attributeValue = userAttributeValueToString(attributeValue);
    logger.userObjectAttributeIsAutoConverted((0, EvaluateLogBuilder_1.formatUserCondition)(condition), key, attributeName, attributeValue);
    return attributeValue;
}
function getUserAttributeValueAsSemVer(attributeName, attributeValue, condition, key, logger) {
    let version;
    if (typeof attributeValue === "string" && (version = (0, Semver_1.parse)(attributeValue.trim()))) {
        return version;
    }
    return handleInvalidUserAttribute(logger, condition, key, attributeName, `'${attributeValue}' is not a valid semantic version`);
}
function getUserAttributeValueAsNumber(attributeName, attributeValue, condition, key, logger) {
    if (typeof attributeValue === "number") {
        return attributeValue;
    }
    let number;
    if (typeof attributeValue === "string"
        && (!isNaN(number = (0, Utils_1.parseFloatStrict)(attributeValue.replace(",", "."))) || attributeValue.trim() === "NaN")) {
        return number;
    }
    return handleInvalidUserAttribute(logger, condition, key, attributeName, `'${attributeValue}' is not a valid decimal number`);
}
function getUserAttributeValueAsUnixTimeSeconds(attributeName, attributeValue, condition, key, logger) {
    if (attributeValue instanceof Date) {
        return attributeValue.getTime() / 1000;
    }
    if (typeof attributeValue === "number") {
        return attributeValue;
    }
    let number;
    if (typeof attributeValue === "string"
        && (!isNaN(number = (0, Utils_1.parseFloatStrict)(attributeValue.replace(",", "."))) || attributeValue.trim() === "NaN")) {
        return number;
    }
    return handleInvalidUserAttribute(logger, condition, key, attributeName, `'${attributeValue}' is not a valid Unix timestamp (number of seconds elapsed since Unix epoch)`);
}
function getUserAttributeValueAsStringArray(attributeName, attributeValue, condition, key, logger) {
    let stringArray = attributeValue;
    if (typeof stringArray === "string") {
        try {
            stringArray = JSON.parse(stringArray);
        }
        catch (_a) { }
    }
    if ((0, Utils_1.isStringArray)(stringArray)) {
        return stringArray;
    }
    return handleInvalidUserAttribute(logger, condition, key, attributeName, `'${attributeValue}' is not a valid string array`);
}
function handleInvalidUserAttribute(logger, condition, key, attributeName, reason) {
    logger.userObjectAttributeIsInvalid((0, EvaluateLogBuilder_1.formatUserCondition)(condition), key, reason, attributeName);
    return invalidUserAttributeError(attributeName, reason);
}
function evaluationDetailsFromEvaluateResult(key, evaluateResult, fetchTime, user) {
    return {
        key,
        value: evaluateResult.selectedValue.value,
        variationId: evaluateResult.selectedValue.variationId,
        fetchTime,
        user,
        isDefaultValue: false,
        matchedTargetingRule: evaluateResult.matchedTargetingRule,
        matchedPercentageOption: evaluateResult.matchedPercentageOption,
    };
}
function evaluationDetailsFromDefaultValue(key, defaultValue, fetchTime, user, errorMessage, errorException) {
    return {
        key,
        value: defaultValue,
        fetchTime,
        user,
        isDefaultValue: true,
        errorMessage,
        errorException
    };
}
function evaluate(evaluator, settings, key, defaultValue, user, remoteConfig, logger) {
    let errorMessage;
    if (!settings) {
        errorMessage = logger.configJsonIsNotPresentSingle(key, "defaultValue", defaultValue).toString();
        return evaluationDetailsFromDefaultValue(key, defaultValue, getTimestampAsDate(remoteConfig), user, errorMessage);
    }
    const setting = settings[key];
    if (!setting) {
        errorMessage = logger.settingEvaluationFailedDueToMissingKey(key, "defaultValue", defaultValue, (0, Utils_1.formatStringList)(Object.keys(settings))).toString();
        return evaluationDetailsFromDefaultValue(key, defaultValue, getTimestampAsDate(remoteConfig), user, errorMessage);
    }
    const evaluateResult = evaluator.evaluate(defaultValue, new EvaluateContext(key, setting, user, settings));
    return evaluationDetailsFromEvaluateResult(key, evaluateResult, getTimestampAsDate(remoteConfig), user);
}
function evaluateAll(evaluator, settings, user, remoteConfig, logger, defaultReturnValue) {
    let errors;
    if (!checkSettingsAvailable(settings, logger, defaultReturnValue)) {
        return [[], errors];
    }
    const evaluationDetailsArray = [];
    for (const [key, setting] of Object.entries(settings)) {
        let evaluationDetails;
        try {
            const evaluateResult = evaluator.evaluate(null, new EvaluateContext(key, setting, user, settings));
            evaluationDetails = evaluationDetailsFromEvaluateResult(key, evaluateResult, getTimestampAsDate(remoteConfig), user);
        }
        catch (err) {
            errors !== null && errors !== void 0 ? errors : (errors = []);
            errors.push(err);
            evaluationDetails = evaluationDetailsFromDefaultValue(key, null, getTimestampAsDate(remoteConfig), user, (0, Utils_1.errorToString)(err), err);
        }
        evaluationDetailsArray.push(evaluationDetails);
    }
    return [evaluationDetailsArray, errors];
}
function checkSettingsAvailable(settings, logger, defaultReturnValue) {
    if (!settings) {
        logger.configJsonIsNotPresent(defaultReturnValue);
        return false;
    }
    return true;
}
function isAllowedValue(value) {
    return typeof value === "boolean" || typeof value === "string" || typeof value === "number";
}
function isCompatibleValue(value, settingType) {
    switch (settingType) {
        case 0: return typeof value === "boolean";
        case 1: return typeof value === "string";
        case 2:
        case 3: return typeof value === "number";
        default: return false;
    }
}
function handleInvalidReturnValue(value) {
    throw new TypeError(value === null ? "Setting value is null." :
        value === void 0 ? "Setting value is undefined." :
            `Setting value '${value}' is of an unsupported type (${typeof value}).`);
}
function getTimestampAsDate(projectConfig) {
    return projectConfig ? new Date(projectConfig.timestamp) : void 0;
}
//# sourceMappingURL=RolloutEvaluator.js.map