import { AutoPollConfigService } from "./AutoPollConfigService.js";
import { AutoPollOptions, LazyLoadOptions, ManualPollOptions } from "./ConfigCatClientOptions.js";
import { RefreshResult } from "./ConfigServiceBase.js";
import { nameOfOverrideBehaviour } from "./FlagOverrides.js";
import { LazyLoadConfigService } from "./LazyLoadConfigService.js";
import { ManualPollConfigService } from "./ManualPollConfigService.js";
import { getWeakRefStub, isWeakRefAvailable } from "./Polyfills.js";
import { RolloutEvaluator, checkSettingsAvailable, evaluate, evaluateAll, evaluationDetailsFromDefaultValue, getTimestampAsDate, handleInvalidReturnValue, isAllowedValue } from "./RolloutEvaluator.js";
import { errorToString, isArray, throwError } from "./Utils.js";
export class ConfigCatClientCache {
    constructor() {
        this.instances = {};
    }
    getOrCreate(options, configCatKernel) {
        let instance;
        const cachedInstance = this.instances[options.sdkKey];
        if (cachedInstance) {
            const [weakRef] = cachedInstance;
            instance = weakRef.deref();
            if (instance) {
                return [instance, true];
            }
        }
        const token = {};
        instance = new ConfigCatClient(options, configCatKernel, token);
        const weakRefCtor = isWeakRefAvailable() ? WeakRef : getWeakRefStub();
        this.instances[options.sdkKey] = [new weakRefCtor(instance), token];
        return [instance, false];
    }
    remove(sdkKey, cacheToken) {
        const cachedInstance = this.instances[sdkKey];
        if (cachedInstance) {
            const [weakRef, token] = cachedInstance;
            const instanceIsAvailable = !!weakRef.deref();
            if (!instanceIsAvailable || token === cacheToken) {
                delete this.instances[sdkKey];
                return instanceIsAvailable;
            }
        }
        return false;
    }
    clear() {
        const removedInstances = [];
        for (const [sdkKey, [weakRef]] of Object.entries(this.instances)) {
            const instance = weakRef.deref();
            if (instance) {
                removedInstances.push(instance);
            }
            delete this.instances[sdkKey];
        }
        return removedInstances;
    }
}
const clientInstanceCache = new ConfigCatClientCache();
export class ConfigCatClient {
    static get instanceCache() { return clientInstanceCache; }
    static get(sdkKey, pollingMode, options, configCatKernel) {
        var _a;
        const invalidSdkKeyError = "Invalid 'sdkKey' value";
        if (!sdkKey) {
            throw new Error(invalidSdkKeyError);
        }
        const optionsClass = pollingMode === 0 ? AutoPollOptions :
            pollingMode === 2 ? ManualPollOptions :
                pollingMode === 1 ? LazyLoadOptions :
                    throwError(new Error("Invalid 'pollingMode' value"));
        const actualOptions = new optionsClass(sdkKey, configCatKernel.sdkType, configCatKernel.sdkVersion, options, configCatKernel.defaultCacheFactory, configCatKernel.eventEmitterFactory);
        if (((_a = actualOptions.flagOverrides) === null || _a === void 0 ? void 0 : _a.behaviour) !== 0 && !isValidSdkKey(sdkKey, actualOptions.baseUrlOverriden)) {
            throw new Error(invalidSdkKeyError);
        }
        const [instance, instanceAlreadyCreated] = clientInstanceCache.getOrCreate(actualOptions, configCatKernel);
        if (instanceAlreadyCreated && options) {
            actualOptions.logger.clientIsAlreadyCreated(sdkKey);
        }
        return instance;
    }
    constructor(options, configCatKernel, cacheToken) {
        var _a;
        this.cacheToken = cacheToken;
        this.addListener = this.on;
        this.off = this.removeListener;
        if (!options) {
            throw new Error("Invalid 'options' value");
        }
        this.options = options;
        this.options.logger.debug("Initializing ConfigCatClient. Options: " + JSON.stringify(this.options));
        if (!configCatKernel) {
            throw new Error("Invalid 'configCatKernel' value");
        }
        if (!configCatKernel.configFetcher) {
            throw new Error("Invalid 'configCatKernel.configFetcher' value");
        }
        this.hooks = options.yieldHooks();
        if (options.defaultUser) {
            this.setDefaultUser(options.defaultUser);
        }
        this.evaluator = new RolloutEvaluator(options.logger);
        if (((_a = options.flagOverrides) === null || _a === void 0 ? void 0 : _a.behaviour) !== 0) {
            this.configService =
                options instanceof AutoPollOptions ? new AutoPollConfigService(configCatKernel.configFetcher, options) :
                    options instanceof ManualPollOptions ? new ManualPollConfigService(configCatKernel.configFetcher, options) :
                        options instanceof LazyLoadOptions ? new LazyLoadConfigService(configCatKernel.configFetcher, options) :
                            throwError(new Error("Invalid 'options' value"));
        }
        else {
            this.hooks.emit("clientReady", 1);
        }
        this.suppressFinalize = registerForFinalization(this, { sdkKey: options.sdkKey, cacheToken, configService: this.configService, logger: options.logger });
    }
    static finalize(data) {
        var _a;
        (_a = data.logger) === null || _a === void 0 ? void 0 : _a.debug("finalize() called");
        if (data.cacheToken) {
            clientInstanceCache.remove(data.sdkKey, data.cacheToken);
        }
        ConfigCatClient.close(data.configService, data.logger);
    }
    static close(configService, logger, hooks) {
        logger === null || logger === void 0 ? void 0 : logger.debug("close() called");
        hooks === null || hooks === void 0 ? void 0 : hooks.tryDisconnect();
        configService === null || configService === void 0 ? void 0 : configService.dispose();
    }
    dispose() {
        const options = this.options;
        options.logger.debug("dispose() called");
        if (this.cacheToken) {
            clientInstanceCache.remove(options.sdkKey, this.cacheToken);
        }
        ConfigCatClient.close(this.configService, options.logger, this.hooks);
        this.suppressFinalize();
    }
    static disposeAll() {
        const removedInstances = clientInstanceCache.clear();
        let errors;
        for (const instance of removedInstances) {
            try {
                ConfigCatClient.close(instance.configService, instance.options.logger, instance.hooks);
                instance.suppressFinalize();
            }
            catch (err) {
                errors !== null && errors !== void 0 ? errors : (errors = []);
                errors.push(err);
            }
        }
        if (errors) {
            throw typeof AggregateError !== "undefined" ? new AggregateError(errors) : errors.pop();
        }
    }
    async getValueAsync(key, defaultValue, user) {
        this.options.logger.debug("getValueAsync() called.");
        validateKey(key);
        ensureAllowedDefaultValue(defaultValue);
        let value, evaluationDetails;
        let remoteConfig = null;
        user !== null && user !== void 0 ? user : (user = this.defaultUser);
        try {
            let settings;
            [settings, remoteConfig] = await this.getSettingsAsync();
            evaluationDetails = evaluate(this.evaluator, settings, key, defaultValue, user, remoteConfig, this.options.logger);
            value = evaluationDetails.value;
        }
        catch (err) {
            this.options.logger.settingEvaluationErrorSingle("getValueAsync", key, "defaultValue", defaultValue, err);
            evaluationDetails = evaluationDetailsFromDefaultValue(key, defaultValue, getTimestampAsDate(remoteConfig), user, errorToString(err), err);
            value = defaultValue;
        }
        this.hooks.emit("flagEvaluated", evaluationDetails);
        return value;
    }
    async getValueDetailsAsync(key, defaultValue, user) {
        this.options.logger.debug("getValueDetailsAsync() called.");
        validateKey(key);
        ensureAllowedDefaultValue(defaultValue);
        let evaluationDetails;
        let remoteConfig = null;
        user !== null && user !== void 0 ? user : (user = this.defaultUser);
        try {
            let settings;
            [settings, remoteConfig] = await this.getSettingsAsync();
            evaluationDetails = evaluate(this.evaluator, settings, key, defaultValue, user, remoteConfig, this.options.logger);
        }
        catch (err) {
            this.options.logger.settingEvaluationErrorSingle("getValueDetailsAsync", key, "defaultValue", defaultValue, err);
            evaluationDetails = evaluationDetailsFromDefaultValue(key, defaultValue, getTimestampAsDate(remoteConfig), user, errorToString(err), err);
        }
        this.hooks.emit("flagEvaluated", evaluationDetails);
        return evaluationDetails;
    }
    async getAllKeysAsync() {
        this.options.logger.debug("getAllKeysAsync() called.");
        const defaultReturnValue = "empty array";
        try {
            const [settings] = await this.getSettingsAsync();
            if (!checkSettingsAvailable(settings, this.options.logger, defaultReturnValue)) {
                return [];
            }
            return Object.keys(settings);
        }
        catch (err) {
            this.options.logger.settingEvaluationError("getAllKeysAsync", defaultReturnValue, err);
            return [];
        }
    }
    async getAllValuesAsync(user) {
        this.options.logger.debug("getAllValuesAsync() called.");
        const defaultReturnValue = "empty array";
        let result, evaluationDetailsArray, evaluationErrors;
        user !== null && user !== void 0 ? user : (user = this.defaultUser);
        try {
            const [settings, remoteConfig] = await this.getSettingsAsync();
            [evaluationDetailsArray, evaluationErrors] = evaluateAll(this.evaluator, settings, user, remoteConfig, this.options.logger, defaultReturnValue);
            result = evaluationDetailsArray.map(details => ({ settingKey: details.key, settingValue: details.value }));
        }
        catch (err) {
            this.options.logger.settingEvaluationError("getAllValuesAsync", defaultReturnValue, err);
            return [];
        }
        if (evaluationErrors === null || evaluationErrors === void 0 ? void 0 : evaluationErrors.length) {
            this.options.logger.settingEvaluationError("getAllValuesAsync", "evaluation result", typeof AggregateError !== "undefined" ? new AggregateError(evaluationErrors) : evaluationErrors.pop());
        }
        for (const evaluationDetail of evaluationDetailsArray) {
            this.hooks.emit("flagEvaluated", evaluationDetail);
        }
        return result;
    }
    async getAllValueDetailsAsync(user) {
        this.options.logger.debug("getAllValueDetailsAsync() called.");
        const defaultReturnValue = "empty array";
        let evaluationDetailsArray, evaluationErrors;
        user !== null && user !== void 0 ? user : (user = this.defaultUser);
        try {
            const [settings, remoteConfig] = await this.getSettingsAsync();
            [evaluationDetailsArray, evaluationErrors] = evaluateAll(this.evaluator, settings, user, remoteConfig, this.options.logger, defaultReturnValue);
        }
        catch (err) {
            this.options.logger.settingEvaluationError("getAllValueDetailsAsync", defaultReturnValue, err);
            return [];
        }
        if (evaluationErrors === null || evaluationErrors === void 0 ? void 0 : evaluationErrors.length) {
            this.options.logger.settingEvaluationError("getAllValueDetailsAsync", "evaluation result", typeof AggregateError !== "undefined" ? new AggregateError(evaluationErrors) : evaluationErrors.pop());
        }
        for (const evaluationDetail of evaluationDetailsArray) {
            this.hooks.emit("flagEvaluated", evaluationDetail);
        }
        return evaluationDetailsArray;
    }
    async getKeyAndValueAsync(variationId) {
        this.options.logger.debug("getKeyAndValueAsync() called.");
        const defaultReturnValue = "null";
        try {
            const [settings] = await this.getSettingsAsync();
            if (!checkSettingsAvailable(settings, this.options.logger, defaultReturnValue)) {
                return null;
            }
            for (const [settingKey, setting] of Object.entries(settings)) {
                if (variationId === setting.variationId) {
                    return { settingKey, settingValue: ensureAllowedValue(setting.value) };
                }
                const { targetingRules } = setting;
                if (targetingRules.length > 0) {
                    for (let i = 0; i < targetingRules.length; i++) {
                        const then = targetingRules[i].then;
                        if (isArray(then)) {
                            for (let j = 0; j < then.length; j++) {
                                const percentageOption = then[j];
                                if (variationId === percentageOption.variationId) {
                                    return { settingKey, settingValue: ensureAllowedValue(percentageOption.value) };
                                }
                            }
                        }
                        else if (variationId === then.variationId) {
                            return { settingKey, settingValue: ensureAllowedValue(then.value) };
                        }
                    }
                }
                const { percentageOptions } = setting;
                if (percentageOptions.length > 0) {
                    for (let i = 0; i < percentageOptions.length; i++) {
                        const percentageOption = percentageOptions[i];
                        if (variationId === percentageOption.variationId) {
                            return { settingKey, settingValue: ensureAllowedValue(percentageOption.value) };
                        }
                    }
                }
            }
            this.options.logger.settingForVariationIdIsNotPresent(variationId);
        }
        catch (err) {
            this.options.logger.settingEvaluationError("getKeyAndValueAsync", defaultReturnValue, err);
        }
        return null;
    }
    async forceRefreshAsync() {
        this.options.logger.debug("forceRefreshAsync() called.");
        if (this.configService) {
            try {
                const [result] = await this.configService.refreshConfigAsync();
                return result;
            }
            catch (err) {
                this.options.logger.forceRefreshError("forceRefreshAsync", err);
                return RefreshResult.failure(errorToString(err), err);
            }
        }
        else {
            return RefreshResult.failure("Client is configured to use the LocalOnly override behavior, which prevents making HTTP requests.");
        }
    }
    setDefaultUser(defaultUser) {
        this.defaultUser = defaultUser;
    }
    clearDefaultUser() {
        this.defaultUser = void 0;
    }
    get isOffline() {
        var _a, _b;
        return (_b = (_a = this.configService) === null || _a === void 0 ? void 0 : _a.isOffline) !== null && _b !== void 0 ? _b : true;
    }
    setOnline() {
        if (this.configService) {
            this.configService.setOnline();
        }
        else {
            this.options.logger.configServiceMethodHasNoEffectDueToOverrideBehavior(nameOfOverrideBehaviour(0), "setOnline");
        }
    }
    setOffline() {
        var _a;
        (_a = this.configService) === null || _a === void 0 ? void 0 : _a.setOffline();
    }
    waitForReady() {
        const configService = this.configService;
        return configService ? configService.readyPromise : Promise.resolve(1);
    }
    snapshot() {
        const getRemoteConfig = () => {
            const config = this.options.cache.getInMemory();
            const settings = !config.isEmpty ? config.config.settings : null;
            return [settings, config];
        };
        let remoteSettings;
        let remoteConfig;
        const { flagOverrides } = this.options;
        if (flagOverrides) {
            const localSettings = flagOverrides.dataSource.getOverridesSync();
            switch (flagOverrides.behaviour) {
                case 0:
                    return new Snapshot(localSettings, null, this);
                case 1:
                    [remoteSettings, remoteConfig] = getRemoteConfig();
                    return new Snapshot(Object.assign(Object.assign({}, (remoteSettings !== null && remoteSettings !== void 0 ? remoteSettings : {})), localSettings), remoteConfig, this);
                case 2:
                    [remoteSettings, remoteConfig] = getRemoteConfig();
                    return new Snapshot(Object.assign(Object.assign({}, localSettings), (remoteSettings !== null && remoteSettings !== void 0 ? remoteSettings : {})), remoteConfig, this);
            }
        }
        [remoteSettings, remoteConfig] = getRemoteConfig();
        return new Snapshot(remoteSettings, remoteConfig, this);
    }
    async getSettingsAsync() {
        this.options.logger.debug("getSettingsAsync() called.");
        const getRemoteConfigAsync = async () => {
            const config = await this.configService.getConfig();
            const settings = !config.isEmpty ? config.config.settings : null;
            return [settings, config];
        };
        const { flagOverrides } = this.options;
        if (flagOverrides) {
            let remoteSettings;
            let remoteConfig;
            const localSettings = await flagOverrides.dataSource.getOverrides();
            switch (flagOverrides.behaviour) {
                case 0:
                    return [localSettings, null];
                case 1:
                    [remoteSettings, remoteConfig] = await getRemoteConfigAsync();
                    return [Object.assign(Object.assign({}, (remoteSettings !== null && remoteSettings !== void 0 ? remoteSettings : {})), localSettings), remoteConfig];
                case 2:
                    [remoteSettings, remoteConfig] = await getRemoteConfigAsync();
                    return [Object.assign(Object.assign({}, localSettings), (remoteSettings !== null && remoteSettings !== void 0 ? remoteSettings : {})), remoteConfig];
            }
        }
        return await getRemoteConfigAsync();
    }
    on(eventName, listener) {
        this.hooks.on(eventName, listener);
        return this;
    }
    once(eventName, listener) {
        this.hooks.once(eventName, listener);
        return this;
    }
    removeListener(eventName, listener) {
        this.hooks.removeListener(eventName, listener);
        return this;
    }
    removeAllListeners(eventName) {
        this.hooks.removeAllListeners(eventName);
        return this;
    }
    listeners(eventName) {
        return this.hooks.listeners(eventName);
    }
    listenerCount(eventName) {
        return this.hooks.listenerCount(eventName);
    }
    eventNames() {
        return this.hooks.eventNames();
    }
}
class Snapshot {
    constructor(mergedSettings, remoteConfig, client) {
        this.mergedSettings = mergedSettings;
        this.remoteConfig = remoteConfig;
        this.defaultUser = client["defaultUser"];
        this.evaluator = client["evaluator"];
        this.options = client["options"];
        this.cacheState = remoteConfig
            ? client["configService"].getCacheState(remoteConfig)
            : 1;
    }
    get fetchedConfig() {
        const config = this.remoteConfig;
        return config && !config.isEmpty ? config.config : null;
    }
    getAllKeys() { return this.mergedSettings ? Object.keys(this.mergedSettings) : []; }
    getValue(key, defaultValue, user) {
        this.options.logger.debug("Snapshot.getValue() called.");
        validateKey(key);
        ensureAllowedDefaultValue(defaultValue);
        let value, evaluationDetails;
        user !== null && user !== void 0 ? user : (user = this.defaultUser);
        try {
            evaluationDetails = evaluate(this.evaluator, this.mergedSettings, key, defaultValue, user, this.remoteConfig, this.options.logger);
            value = evaluationDetails.value;
        }
        catch (err) {
            this.options.logger.settingEvaluationErrorSingle("Snapshot.getValue", key, "defaultValue", defaultValue, err);
            evaluationDetails = evaluationDetailsFromDefaultValue(key, defaultValue, getTimestampAsDate(this.remoteConfig), user, errorToString(err), err);
            value = defaultValue;
        }
        this.options.hooks.emit("flagEvaluated", evaluationDetails);
        return value;
    }
    getValueDetails(key, defaultValue, user) {
        this.options.logger.debug("Snapshot.getValueDetails() called.");
        validateKey(key);
        ensureAllowedDefaultValue(defaultValue);
        let evaluationDetails;
        user !== null && user !== void 0 ? user : (user = this.defaultUser);
        try {
            evaluationDetails = evaluate(this.evaluator, this.mergedSettings, key, defaultValue, user, this.remoteConfig, this.options.logger);
        }
        catch (err) {
            this.options.logger.settingEvaluationErrorSingle("Snapshot.getValueDetails", key, "defaultValue", defaultValue, err);
            evaluationDetails = evaluationDetailsFromDefaultValue(key, defaultValue, getTimestampAsDate(this.remoteConfig), user, errorToString(err), err);
        }
        this.options.hooks.emit("flagEvaluated", evaluationDetails);
        return evaluationDetails;
    }
}
function isValidSdkKey(sdkKey, customBaseUrl) {
    const proxyPrefix = "configcat-proxy/";
    if (customBaseUrl && sdkKey.length > proxyPrefix.length && sdkKey.lastIndexOf(proxyPrefix, 0) === 0) {
        return true;
    }
    const components = sdkKey.split("/");
    const keyLength = 22;
    switch (components.length) {
        case 2: return components[0].length === keyLength && components[1].length === keyLength;
        case 3: return components[0] === "configcat-sdk-1" && components[1].length === keyLength && components[2].length === keyLength;
        default: return false;
    }
}
function validateKey(key) {
    if (!key) {
        throw new Error("Invalid 'key' value");
    }
}
function ensureAllowedDefaultValue(value) {
    if (value != null && !isAllowedValue(value)) {
        throw new TypeError("The default value must be boolean, number, string, null or undefined.");
    }
}
function ensureAllowedValue(value) {
    return isAllowedValue(value) ? value : handleInvalidReturnValue(value);
}
let registerForFinalization = function (client, data) {
    if (typeof FinalizationRegistry !== "undefined") {
        const finalizationRegistry = new FinalizationRegistry(data => ConfigCatClient["finalize"](data));
        registerForFinalization = (client, data) => {
            const unregisterToken = {};
            finalizationRegistry.register(client, data, unregisterToken);
            return () => finalizationRegistry.unregister(unregisterToken);
        };
    }
    else {
        registerForFinalization = () => () => { };
    }
    return registerForFinalization(client, data);
};
//# sourceMappingURL=ConfigCatClient.js.map