import { ExternalConfigCache, InMemoryConfigCache } from "./ConfigCatCache.js";
import { ConfigCatConsoleLogger, LoggerWrapper } from "./ConfigCatLogger.js";
import { NullEventEmitter } from "./EventEmitter.js";
import { sha1 } from "./Hash.js";
import { Hooks } from "./Hooks.js";
import { getWeakRefStub, isWeakRefAvailable } from "./Polyfills.js";
import { ProjectConfig } from "./ProjectConfig.js";
export var PollingMode;
(function (PollingMode) {
    PollingMode[PollingMode["AutoPoll"] = 0] = "AutoPoll";
    PollingMode[PollingMode["LazyLoad"] = 1] = "LazyLoad";
    PollingMode[PollingMode["ManualPoll"] = 2] = "ManualPoll";
})(PollingMode || (PollingMode = {}));
export var DataGovernance;
(function (DataGovernance) {
    DataGovernance[DataGovernance["Global"] = 0] = "Global";
    DataGovernance[DataGovernance["EuOnly"] = 1] = "EuOnly";
})(DataGovernance || (DataGovernance = {}));
export class OptionsBase {
    constructor(sdkKey, clientVersion, options, defaultCacheFactory, eventEmitterFactory) {
        var _a, _b, _c;
        this.requestTimeoutMs = 30000;
        this.baseUrlOverriden = false;
        this.offline = false;
        if (!sdkKey) {
            throw new Error("Invalid 'sdkKey' value");
        }
        this.sdkKey = sdkKey;
        this.clientVersion = clientVersion;
        this.dataGovernance = (_a = options === null || options === void 0 ? void 0 : options.dataGovernance) !== null && _a !== void 0 ? _a : 0;
        switch (this.dataGovernance) {
            case 1:
                this.baseUrl = "https://cdn-eu.configcat.com";
                break;
            default:
                this.baseUrl = "https://cdn-global.configcat.com";
                break;
        }
        const eventEmitter = (_b = eventEmitterFactory === null || eventEmitterFactory === void 0 ? void 0 : eventEmitterFactory()) !== null && _b !== void 0 ? _b : new NullEventEmitter();
        const hooks = new Hooks(eventEmitter);
        const hooksWeakRef = new (isWeakRefAvailable() ? WeakRef : getWeakRefStub())(hooks);
        this.hooks = {
            hooks,
            hooksWeakRef,
            emit(eventName, ...args) {
                var _a, _b;
                return (_b = (_a = this.hooksWeakRef.deref()) === null || _a === void 0 ? void 0 : _a.emit(eventName, ...args)) !== null && _b !== void 0 ? _b : false;
            }
        };
        let logger;
        let cache;
        if (options) {
            logger = options.logger;
            cache = options.cache;
            if (options.requestTimeoutMs) {
                if (options.requestTimeoutMs < 0) {
                    throw new Error("Invalid 'requestTimeoutMs' value");
                }
                this.requestTimeoutMs = options.requestTimeoutMs;
            }
            if (options.baseUrl) {
                this.baseUrl = options.baseUrl;
                this.baseUrlOverriden = true;
            }
            if (options.flagOverrides) {
                this.flagOverrides = options.flagOverrides;
            }
            if (options.defaultUser) {
                this.defaultUser = options.defaultUser;
            }
            if (options.offline) {
                this.offline = options.offline;
            }
            (_c = options.setupHooks) === null || _c === void 0 ? void 0 : _c.call(options, hooks);
        }
        this.logger = new LoggerWrapper(logger !== null && logger !== void 0 ? logger : new ConfigCatConsoleLogger(), this.hooks);
        this.cache = cache
            ? new ExternalConfigCache(cache, this.logger)
            : (defaultCacheFactory ? defaultCacheFactory(this) : new InMemoryConfigCache());
    }
    yieldHooks() {
        const hooksWrapper = this.hooks;
        const hooks = hooksWrapper.hooks;
        delete hooksWrapper.hooks;
        return hooks !== null && hooks !== void 0 ? hooks : new Hooks(new NullEventEmitter());
    }
    getUrl() {
        return this.baseUrl + "/configuration-files/" + this.sdkKey + "/" + OptionsBase.configFileName + "?sdk=" + this.clientVersion;
    }
    getCacheKey() {
        return sha1(`${this.sdkKey}_${OptionsBase.configFileName}_${ProjectConfig.serializationFormatVersion}`);
    }
}
OptionsBase.configFileName = "config_v6.json";
export class AutoPollOptions extends OptionsBase {
    constructor(sdkKey, sdkType, sdkVersion, options, defaultCacheFactory, eventEmitterFactory) {
        super(sdkKey, sdkType + "/a-" + sdkVersion, options, defaultCacheFactory, eventEmitterFactory);
        this.pollIntervalSeconds = 60;
        this.maxInitWaitTimeSeconds = 5;
        if (options) {
            if (options.pollIntervalSeconds != null) {
                this.pollIntervalSeconds = options.pollIntervalSeconds;
            }
            if (options.maxInitWaitTimeSeconds != null) {
                this.maxInitWaitTimeSeconds = options.maxInitWaitTimeSeconds;
            }
        }
        const maxSetTimeoutIntervalSecs = 2147483;
        if (!(typeof this.pollIntervalSeconds === "number" && 1 <= this.pollIntervalSeconds && this.pollIntervalSeconds <= maxSetTimeoutIntervalSecs)) {
            throw new Error("Invalid 'pollIntervalSeconds' value");
        }
        if (!(typeof this.maxInitWaitTimeSeconds === "number" && this.maxInitWaitTimeSeconds <= maxSetTimeoutIntervalSecs)) {
            throw new Error("Invalid 'maxInitWaitTimeSeconds' value");
        }
    }
}
export class ManualPollOptions extends OptionsBase {
    constructor(sdkKey, sdkType, sdkVersion, options, defaultCacheFactory, eventEmitterFactory) {
        super(sdkKey, sdkType + "/m-" + sdkVersion, options, defaultCacheFactory, eventEmitterFactory);
    }
}
export class LazyLoadOptions extends OptionsBase {
    constructor(sdkKey, sdkType, sdkVersion, options, defaultCacheFactory, eventEmitterFactory) {
        super(sdkKey, sdkType + "/l-" + sdkVersion, options, defaultCacheFactory, eventEmitterFactory);
        this.cacheTimeToLiveSeconds = 60;
        if (options) {
            if (options.cacheTimeToLiveSeconds != null) {
                this.cacheTimeToLiveSeconds = options.cacheTimeToLiveSeconds;
            }
        }
        if (!(typeof this.cacheTimeToLiveSeconds === "number" && 1 <= this.cacheTimeToLiveSeconds && this.cacheTimeToLiveSeconds <= 2147483647)) {
            throw new Error("Invalid 'cacheTimeToLiveSeconds' value");
        }
    }
}
//# sourceMappingURL=ConfigCatClientOptions.js.map