import { FetchError, FetchResult } from "./ConfigFetcher.js";
import { Config, ProjectConfig } from "./ProjectConfig.js";
export class RefreshResult {
    constructor(errorMessage, errorException) {
        this.errorMessage = errorMessage;
        this.errorException = errorException;
    }
    get isSuccess() { return this.errorMessage === null; }
    static from(fetchResult) {
        return fetchResult.status !== 2
            ? RefreshResult.success()
            : RefreshResult.failure(fetchResult.errorMessage, fetchResult.errorException);
    }
    static success() {
        return new RefreshResult(null);
    }
    static failure(errorMessage, errorException) {
        return new RefreshResult(errorMessage, errorException);
    }
}
export var ClientCacheState;
(function (ClientCacheState) {
    ClientCacheState[ClientCacheState["NoFlagData"] = 0] = "NoFlagData";
    ClientCacheState[ClientCacheState["HasLocalOverrideFlagDataOnly"] = 1] = "HasLocalOverrideFlagDataOnly";
    ClientCacheState[ClientCacheState["HasCachedFlagDataOnly"] = 2] = "HasCachedFlagDataOnly";
    ClientCacheState[ClientCacheState["HasUpToDateFlagData"] = 3] = "HasUpToDateFlagData";
})(ClientCacheState || (ClientCacheState = {}));
var ConfigServiceStatus;
(function (ConfigServiceStatus) {
    ConfigServiceStatus[ConfigServiceStatus["Online"] = 0] = "Online";
    ConfigServiceStatus[ConfigServiceStatus["Offline"] = 1] = "Offline";
    ConfigServiceStatus[ConfigServiceStatus["Disposed"] = 2] = "Disposed";
})(ConfigServiceStatus || (ConfigServiceStatus = {}));
function nameOfConfigServiceStatus(value) {
    return ConfigServiceStatus[value];
}
export class ConfigServiceBase {
    constructor(configFetcher, options) {
        this.configFetcher = configFetcher;
        this.options = options;
        this.pendingFetch = null;
        this.cacheKey = options.getCacheKey();
        this.configFetcher = configFetcher;
        this.options = options;
        this.status = options.offline ? 1 : 0;
    }
    dispose() {
        this.status = 2;
    }
    get disposed() {
        return this.status === 2;
    }
    async refreshConfigAsync() {
        const latestConfig = await this.options.cache.get(this.cacheKey);
        if (!this.isOffline) {
            const [fetchResult, config] = await this.refreshConfigCoreAsync(latestConfig);
            return [RefreshResult.from(fetchResult), config];
        }
        else {
            const errorMessage = this.options.logger.configServiceCannotInitiateHttpCalls().toString();
            return [RefreshResult.failure(errorMessage), latestConfig];
        }
    }
    async refreshConfigCoreAsync(latestConfig) {
        const fetchResult = await this.fetchAsync(latestConfig);
        let configChanged = false;
        const success = fetchResult.status === 0;
        if (success
            || fetchResult.config.timestamp > latestConfig.timestamp && (!fetchResult.config.isEmpty || latestConfig.isEmpty)) {
            await this.options.cache.set(this.cacheKey, fetchResult.config);
            configChanged = success && !ProjectConfig.equals(fetchResult.config, latestConfig);
            latestConfig = fetchResult.config;
        }
        this.onConfigFetched(fetchResult.config);
        if (configChanged) {
            this.onConfigChanged(fetchResult.config);
        }
        return [fetchResult, latestConfig];
    }
    onConfigFetched(newConfig) {
        this.options.logger.debug("config fetched");
    }
    onConfigChanged(newConfig) {
        var _a;
        this.options.logger.debug("config changed");
        this.options.hooks.emit("configChanged", (_a = newConfig.config) !== null && _a !== void 0 ? _a : new Config({}));
    }
    fetchAsync(lastConfig) {
        var _a;
        return (_a = this.pendingFetch) !== null && _a !== void 0 ? _a : (this.pendingFetch = (async () => {
            try {
                return await this.fetchLogicAsync(lastConfig);
            }
            finally {
                this.pendingFetch = null;
            }
        })());
    }
    async fetchLogicAsync(lastConfig) {
        var _a;
        const options = this.options;
        options.logger.debug("ConfigServiceBase.fetchLogicAsync() - called.");
        let errorMessage;
        try {
            const [response, configOrError] = await this.fetchRequestAsync((_a = lastConfig.httpETag) !== null && _a !== void 0 ? _a : null);
            switch (response.statusCode) {
                case 200:
                    if (!(configOrError instanceof Config)) {
                        errorMessage = options.logger.fetchReceived200WithInvalidBody(configOrError).toString();
                        options.logger.debug(`ConfigServiceBase.fetchLogicAsync(): ${response.statusCode} ${response.reasonPhrase} was received but the HTTP response content was invalid. Returning null.`);
                        return FetchResult.error(lastConfig, errorMessage, configOrError);
                    }
                    options.logger.debug("ConfigServiceBase.fetchLogicAsync(): fetch was successful. Returning new config.");
                    return FetchResult.success(new ProjectConfig(response.body, configOrError, ProjectConfig.generateTimestamp(), response.eTag));
                case 304:
                    if (lastConfig.isEmpty) {
                        errorMessage = options.logger.fetchReceived304WhenLocalCacheIsEmpty(response.statusCode, response.reasonPhrase).toString();
                        options.logger.debug(`ConfigServiceBase.fetchLogicAsync(): ${response.statusCode} ${response.reasonPhrase} was received when no config is cached locally. Returning null.`);
                        return FetchResult.error(lastConfig, errorMessage);
                    }
                    options.logger.debug("ConfigServiceBase.fetchLogicAsync(): content was not modified. Returning last config with updated timestamp.");
                    return FetchResult.notModified(lastConfig.with(ProjectConfig.generateTimestamp()));
                case 403:
                case 404:
                    errorMessage = options.logger.fetchFailedDueToInvalidSdkKey().toString();
                    options.logger.debug("ConfigServiceBase.fetchLogicAsync(): fetch was unsuccessful. Returning last config (if any) with updated timestamp.");
                    return FetchResult.error(lastConfig.with(ProjectConfig.generateTimestamp()), errorMessage);
                default:
                    errorMessage = options.logger.fetchFailedDueToUnexpectedHttpResponse(response.statusCode, response.reasonPhrase).toString();
                    options.logger.debug("ConfigServiceBase.fetchLogicAsync(): fetch was unsuccessful. Returning null.");
                    return FetchResult.error(lastConfig, errorMessage);
            }
        }
        catch (err) {
            errorMessage = (err instanceof FetchError && err.cause === "timeout"
                ? options.logger.fetchFailedDueToRequestTimeout(err.args[0], err)
                : options.logger.fetchFailedDueToUnexpectedError(err)).toString();
            options.logger.debug("ConfigServiceBase.fetchLogicAsync(): fetch was unsuccessful. Returning null.");
            return FetchResult.error(lastConfig, errorMessage, err);
        }
    }
    async fetchRequestAsync(lastETag, maxRetryCount = 2) {
        const options = this.options;
        options.logger.debug("ConfigServiceBase.fetchRequestAsync() - called.");
        for (let retryNumber = 0;; retryNumber++) {
            options.logger.debug(`ConfigServiceBase.fetchRequestAsync(): calling fetchLogic()${retryNumber > 0 ? `, retry ${retryNumber}/${maxRetryCount}` : ""}`);
            const response = await this.configFetcher.fetchLogic(options, lastETag);
            if (response.statusCode !== 200) {
                return [response];
            }
            if (!response.body) {
                options.logger.debug("ConfigServiceBase.fetchRequestAsync(): no response body.");
                return [response, new Error("No response body.")];
            }
            let config;
            try {
                config = Config.deserialize(response.body);
            }
            catch (err) {
                options.logger.debug("ConfigServiceBase.fetchRequestAsync(): invalid response body.");
                return [response, err];
            }
            const preferences = config.preferences;
            if (!preferences) {
                options.logger.debug("ConfigServiceBase.fetchRequestAsync(): preferences is empty.");
                return [response, config];
            }
            const baseUrl = preferences.baseUrl;
            if (!baseUrl || baseUrl === options.baseUrl) {
                options.logger.debug("ConfigServiceBase.fetchRequestAsync(): baseUrl OK.");
                return [response, config];
            }
            const redirect = preferences.redirectMode;
            if (options.baseUrlOverriden && redirect !== 2) {
                options.logger.debug("ConfigServiceBase.fetchRequestAsync(): options.baseUrlOverriden && redirect !== 2.");
                return [response, config];
            }
            options.baseUrl = baseUrl;
            if (redirect === 0) {
                return [response, config];
            }
            if (redirect === 1) {
                options.logger.dataGovernanceIsOutOfSync();
            }
            if (retryNumber >= maxRetryCount) {
                options.logger.fetchFailedDueToRedirectLoop();
                return [response, config];
            }
        }
    }
    get isOfflineExactly() {
        return this.status === 1;
    }
    get isOffline() {
        return this.status !== 0;
    }
    setOnlineCore() { }
    setOnline() {
        if (this.status === 1) {
            this.setOnlineCore();
            this.status = 0;
            this.options.logger.configServiceStatusChanged(nameOfConfigServiceStatus(this.status));
        }
        else if (this.disposed) {
            this.options.logger.configServiceMethodHasNoEffectDueToDisposedClient("setOnline");
        }
    }
    setOfflineCore() { }
    setOffline() {
        if (this.status === 0) {
            this.setOfflineCore();
            this.status = 1;
            this.options.logger.configServiceStatusChanged(nameOfConfigServiceStatus(this.status));
        }
        else if (this.disposed) {
            this.options.logger.configServiceMethodHasNoEffectDueToDisposedClient("setOffline");
        }
    }
    syncUpWithCache() {
        return this.options.cache.get(this.cacheKey);
    }
    async getReadyPromise(state, waitForReadyAsync) {
        const cacheState = await waitForReadyAsync(state);
        this.options.hooks.emit("clientReady", cacheState);
        return cacheState;
    }
}
//# sourceMappingURL=ConfigServiceBase.js.map