function isSingle(listeners) {
    return !!listeners.fn;
}
export class DefaultEventEmitter {
    constructor() {
        this.events = {};
        this.eventCount = 0;
        this.addListener = this.on;
        this.off = this.removeListener;
    }
    addListenerCore(eventName, fn, once) {
        if (typeof fn !== "function") {
            throw new TypeError("Listener must be a function");
        }
        const listeners = this.events[eventName];
        const listener = { fn, once };
        if (!listeners) {
            this.events[eventName] = listener;
            this.eventCount++;
        }
        else if (isSingle(listeners)) {
            this.events[eventName] = [listeners, listener];
        }
        else {
            listeners.push(listener);
        }
        return this;
    }
    removeListenerCore(eventName, state, isMatch) {
        const listeners = this.events[eventName];
        if (!listeners) {
            return this;
        }
        if (!isSingle(listeners)) {
            for (let i = listeners.length - 1; i >= 0; i--) {
                if (isMatch(listeners[i], state)) {
                    listeners.splice(i, 1);
                    if (!listeners.length) {
                        this.removeEvent(eventName);
                    }
                    else if (listeners.length === 1) {
                        this.events[eventName] = listeners[0];
                    }
                    break;
                }
            }
        }
        else if (isMatch(listeners, state)) {
            this.removeEvent(eventName);
        }
        return this;
    }
    removeEvent(eventName) {
        if (--this.eventCount === 0) {
            this.events = {};
        }
        else {
            delete this.events[eventName];
        }
    }
    on(eventName, listener) {
        return this.addListenerCore(eventName, listener, false);
    }
    once(eventName, listener) {
        return this.addListenerCore(eventName, listener, true);
    }
    removeListener(eventName, listener) {
        if (typeof listener !== "function") {
            throw new TypeError("Listener must be a function");
        }
        return this.removeListenerCore(eventName, listener, (listener, fn) => listener.fn === fn);
    }
    removeAllListeners(eventName) {
        if (!eventName) {
            this.events = {};
            this.eventCount = 0;
        }
        else if (this.events[eventName]) {
            this.removeEvent(eventName);
        }
        return this;
    }
    listeners(eventName) {
        const listeners = this.events[eventName];
        if (!listeners) {
            return [];
        }
        if (isSingle(listeners)) {
            return [listeners.fn];
        }
        const length = listeners.length, fns = new Array(length);
        for (let i = 0; i < length; i++) {
            fns[i] = listeners[i].fn;
        }
        return fns;
    }
    listenerCount(eventName) {
        const listeners = this.events[eventName];
        if (!listeners) {
            return 0;
        }
        if (isSingle(listeners)) {
            return 1;
        }
        return listeners.length;
    }
    eventNames() {
        const names = [];
        if (this.eventCount === 0) {
            return names;
        }
        const events = this.events;
        for (const name in events) {
            if (Object.prototype.hasOwnProperty.call(events, name)) {
                names.push(name);
            }
        }
        if (Object.getOwnPropertySymbols) {
            return names.concat(Object.getOwnPropertySymbols(events));
        }
        return names;
    }
    emit(eventName, arg0, arg1, arg2, arg3, ...moreArgs) {
        let listeners = this.events[eventName];
        if (!listeners) {
            return false;
        }
        let listener, length;
        if (isSingle(listeners)) {
            [listener, length] = [listeners, 1];
        }
        else {
            listeners = listeners.slice();
            [listener, length] = [listeners[0], listeners.length];
        }
        const argCount = arguments.length - 1;
        for (let i = 0;;) {
            if (listener.once) {
                this.removeListenerCore(eventName, listener, (listener, toRemove) => listener === toRemove);
            }
            switch (argCount) {
                case 0:
                    listener.fn.call(this);
                    break;
                case 1:
                    listener.fn.call(this, arg0);
                    break;
                case 2:
                    listener.fn.call(this, arg0, arg1);
                    break;
                case 3:
                    listener.fn.call(this, arg0, arg1, arg2);
                    break;
                case 4:
                    listener.fn.call(this, arg0, arg1, arg2, arg3);
                    break;
                default:
                    const args = new Array(argCount);
                    for (let j = 0; j < argCount; j++) {
                        args[j] = arguments[j + 1];
                    }
                    listener.fn.apply(this, args);
                    break;
            }
            if (++i >= length) {
                break;
            }
            listener = listeners[i];
        }
        return true;
    }
}
//# sourceMappingURL=DefaultEventEmitter.js.map