import { isAllowedValue } from "./RolloutEvaluator.js";
import { formatStringList, isArray, isStringArray } from "./Utils.js";
const invalidValuePlaceholder = "<invalid value>";
const invalidNamePlaceholder = "<invalid name>";
const invalidOperatorPlaceholder = "<invalid operator>";
const invalidReferencePlaceholder = "<invalid reference>";
const stringListMaxLength = 10;
export class EvaluateLogBuilder {
    constructor(eol) {
        this.eol = eol;
        this.log = "";
        this.indent = "";
    }
    resetIndent() {
        this.indent = "";
        return this;
    }
    increaseIndent() {
        this.indent += "  ";
        return this;
    }
    decreaseIndent() {
        this.indent = this.indent.slice(0, -2);
        return this;
    }
    newLine(text) {
        this.log += this.eol + this.indent + (text !== null && text !== void 0 ? text : "");
        return this;
    }
    append(text) {
        this.log += text;
        return this;
    }
    toString() {
        return this.log;
    }
    appendUserConditionCore(comparisonAttribute, comparator, comparisonValue) {
        return this.append(`User.${comparisonAttribute} ${formatUserComparator(comparator)} '${comparisonValue !== null && comparisonValue !== void 0 ? comparisonValue : invalidValuePlaceholder}'`);
    }
    appendUserConditionString(comparisonAttribute, comparator, comparisonValue, isSensitive) {
        if (typeof comparisonValue !== "string") {
            return this.appendUserConditionCore(comparisonAttribute, comparator);
        }
        return this.appendUserConditionCore(comparisonAttribute, comparator, !isSensitive ? comparisonValue : "<hashed value>");
    }
    appendUserConditionStringList(comparisonAttribute, comparator, comparisonValue, isSensitive) {
        if (!isStringArray(comparisonValue)) {
            return this.appendUserConditionCore(comparisonAttribute, comparator);
        }
        const valueText = "value", valuesText = "values";
        const comparatorFormatted = formatUserComparator(comparator);
        if (isSensitive) {
            return this.append(`User.${comparisonAttribute} ${comparatorFormatted} [<${comparisonValue.length} hashed ${comparisonValue.length === 1 ? valueText : valuesText}>]`);
        }
        else {
            const comparisonValueFormatted = formatStringList(comparisonValue, stringListMaxLength, count => `, ... <${count} more ${count === 1 ? valueText : valuesText}>`);
            return this.append(`User.${comparisonAttribute} ${comparatorFormatted} [${comparisonValueFormatted}]`);
        }
    }
    appendUserConditionNumber(comparisonAttribute, comparator, comparisonValue, isDateTime) {
        if (typeof comparisonValue !== "number") {
            return this.appendUserConditionCore(comparisonAttribute, comparator);
        }
        const comparatorFormatted = formatUserComparator(comparator);
        let date;
        return isDateTime && !isNaN((date = new Date(comparisonValue * 1000)))
            ? this.append(`User.${comparisonAttribute} ${comparatorFormatted} '${comparisonValue}' (${date.toISOString()} UTC)`)
            : this.append(`User.${comparisonAttribute} ${comparatorFormatted} '${comparisonValue}'`);
    }
    appendUserCondition(condition) {
        const comparisonAttribute = typeof condition.comparisonAttribute === "string" ? condition.comparisonAttribute : invalidNamePlaceholder;
        const comparator = condition.comparator;
        switch (condition.comparator) {
            case 0:
            case 1:
            case 2:
            case 3:
            case 4:
            case 5:
            case 30:
            case 31:
            case 32:
            case 33:
            case 34:
            case 35:
                return this.appendUserConditionStringList(comparisonAttribute, comparator, condition.comparisonValue, false);
            case 6:
            case 7:
            case 8:
            case 9:
            case 28:
            case 29:
                return this.appendUserConditionString(comparisonAttribute, comparator, condition.comparisonValue, false);
            case 10:
            case 11:
            case 12:
            case 13:
            case 14:
            case 15:
                return this.appendUserConditionNumber(comparisonAttribute, comparator, condition.comparisonValue);
            case 16:
            case 17:
            case 22:
            case 23:
            case 24:
            case 25:
            case 26:
            case 27:
                return this.appendUserConditionStringList(comparisonAttribute, comparator, condition.comparisonValue, true);
            case 18:
            case 19:
                return this.appendUserConditionNumber(comparisonAttribute, comparator, condition.comparisonValue, true);
            case 20:
            case 21:
                return this.appendUserConditionString(comparisonAttribute, comparator, condition.comparisonValue, true);
            default:
                return this.appendUserConditionCore(comparisonAttribute, comparator, condition.comparisonValue);
        }
    }
    appendPrerequisiteFlagCondition(condition, settings) {
        const prerequisiteFlagKey = typeof condition.prerequisiteFlagKey !== "string" ? invalidNamePlaceholder :
            !(condition.prerequisiteFlagKey in settings) ? invalidReferencePlaceholder :
                condition.prerequisiteFlagKey;
        const comparator = condition.comparator;
        const comparisonValue = condition.comparisonValue;
        return this.append(`Flag '${prerequisiteFlagKey}' ${formatPrerequisiteFlagComparator(comparator)} '${valueToString(comparisonValue)}'`);
    }
    appendSegmentCondition(condition) {
        const segment = condition.segment;
        const comparator = condition.comparator;
        const segmentName = segment == null ? invalidReferencePlaceholder :
            typeof segment.name !== "string" || !segment.name ? invalidNamePlaceholder :
                segment.name;
        return this.append(`User ${formatSegmentComparator(comparator)} '${segmentName}'`);
    }
    appendConditionResult(result) {
        return this.append(`${result}`);
    }
    appendConditionConsequence(result) {
        this.append(" => ").appendConditionResult(result);
        return result ? this : this.append(", skipping the remaining AND conditions");
    }
    appendTargetingRuleThenPart(targetingRule, newLine) {
        (newLine ? this.newLine() : this.append(" "))
            .append("THEN");
        const then = targetingRule.then;
        return this.append(!isArray(then) ? ` '${valueToString(then.value)}'` : " % options");
    }
    appendTargetingRuleConsequence(targetingRule, isMatchOrError, newLine) {
        this.increaseIndent();
        this.appendTargetingRuleThenPart(targetingRule, newLine)
            .append(" => ").append(isMatchOrError === true ? "MATCH, applying rule" : isMatchOrError === false ? "no match" : isMatchOrError);
        return this.decreaseIndent();
    }
}
export function formatUserComparator(comparator) {
    switch (comparator) {
        case 0:
        case 16:
        case 4: return "IS ONE OF";
        case 1:
        case 17:
        case 5: return "IS NOT ONE OF";
        case 2: return "CONTAINS ANY OF";
        case 3: return "NOT CONTAINS ANY OF";
        case 6:
        case 12: return "<";
        case 7:
        case 13: return "<=";
        case 8:
        case 14: return ">";
        case 9:
        case 15: return ">=";
        case 10: return "=";
        case 11: return "!=";
        case 18: return "BEFORE";
        case 19: return "AFTER";
        case 28:
        case 20: return "EQUALS";
        case 29:
        case 21: return "NOT EQUALS";
        case 30:
        case 22: return "STARTS WITH ANY OF";
        case 31:
        case 23: return "NOT STARTS WITH ANY OF";
        case 32:
        case 24: return "ENDS WITH ANY OF";
        case 33:
        case 25: return "NOT ENDS WITH ANY OF";
        case 34:
        case 26: return "ARRAY CONTAINS ANY OF";
        case 35:
        case 27: return "ARRAY NOT CONTAINS ANY OF";
        default: return invalidOperatorPlaceholder;
    }
}
export function formatUserCondition(condition) {
    return new EvaluateLogBuilder("").appendUserCondition(condition).toString();
}
export function formatPrerequisiteFlagComparator(comparator) {
    switch (comparator) {
        case 0: return "EQUALS";
        case 1: return "NOT EQUALS";
        default: return invalidOperatorPlaceholder;
    }
}
export function formatSegmentComparator(comparator) {
    switch (comparator) {
        case 0: return "IS IN SEGMENT";
        case 1: return "IS NOT IN SEGMENT";
        default: return invalidOperatorPlaceholder;
    }
}
export function valueToString(value) {
    return isAllowedValue(value) ? value.toString() : invalidValuePlaceholder;
}
//# sourceMappingURL=EvaluateLogBuilder.js.map