import { Setting } from "./ProjectConfig.js";
import { isArray, parseFloatStrict } from "./Utils.js";
export var OverrideBehaviour;
(function (OverrideBehaviour) {
    OverrideBehaviour[OverrideBehaviour["LocalOnly"] = 0] = "LocalOnly";
    OverrideBehaviour[OverrideBehaviour["LocalOverRemote"] = 1] = "LocalOverRemote";
    OverrideBehaviour[OverrideBehaviour["RemoteOverLocal"] = 2] = "RemoteOverLocal";
})(OverrideBehaviour || (OverrideBehaviour = {}));
export function nameOfOverrideBehaviour(value) {
    return OverrideBehaviour[value];
}
export class MapOverrideDataSource {
    constructor(map, watchChanges) {
        this.initialSettings = getSettingsFromMap(map);
        if (watchChanges) {
            this.map = map;
        }
    }
    getOverrides() {
        return Promise.resolve(this.getOverridesSync());
    }
    getOverridesSync() {
        return this.map
            ? getSettingsFromMap(this.map)
            : this.initialSettings;
    }
}
function getSettingsFromMap(map) {
    const settings = {};
    for (const key in map) {
        if (Object.prototype.hasOwnProperty.call(map, key)) {
            settings[key] = Setting.fromValue(map[key]);
        }
    }
    return settings;
}
const DEFAULT_PARAM_PREFIX = "cc-";
const FORCE_STRING_VALUE_SUFFIX = ";str";
class DefaultQueryStringProvider {
    get currentValue() {
        if (typeof location === "undefined")
            return;
        return location.search;
    }
}
let defaultQueryStringProvider;
export class QueryParamsOverrideDataSource {
    constructor(watchChanges, paramPrefix, queryStringProvider) {
        this.watchChanges = watchChanges;
        this.paramPrefix = paramPrefix !== null && paramPrefix !== void 0 ? paramPrefix : DEFAULT_PARAM_PREFIX;
        queryStringProvider !== null && queryStringProvider !== void 0 ? queryStringProvider : (queryStringProvider = defaultQueryStringProvider !== null && defaultQueryStringProvider !== void 0 ? defaultQueryStringProvider : (defaultQueryStringProvider = new DefaultQueryStringProvider()));
        this.queryStringProvider = queryStringProvider;
        const currentQueryStringOrParams = queryStringProvider.currentValue;
        this.settings = getSettingsFromQueryString(currentQueryStringOrParams, this.paramPrefix);
        this.queryString = getQueryString(currentQueryStringOrParams);
    }
    getOverrides() {
        return Promise.resolve(this.getOverridesSync());
    }
    getOverridesSync() {
        if (this.watchChanges) {
            const currentQueryStringOrParams = this.queryStringProvider.currentValue;
            const currentQueryString = getQueryString(currentQueryStringOrParams);
            if (this.queryString !== currentQueryString) {
                this.settings = getSettingsFromQueryString(currentQueryStringOrParams, this.paramPrefix);
                this.queryString = currentQueryString;
            }
        }
        return this.settings;
    }
}
function getQueryString(queryStringOrParams) {
    if (queryStringOrParams == null) {
        return "";
    }
    if (typeof queryStringOrParams === "string") {
        return queryStringOrParams;
    }
    let queryString = "", separator = "?";
    for (const key in queryStringOrParams) {
        if (!Object.prototype.hasOwnProperty.call(queryStringOrParams, key))
            continue;
        const values = queryStringOrParams[key];
        let value, length;
        if (!isArray(values))
            value = values, length = 1;
        else if (values.length)
            value = values[0], length = values.length;
        else
            continue;
        for (let i = 0;;) {
            queryString += separator + encodeURIComponent(key) + "=" + encodeURIComponent(value);
            if (++i >= length)
                break;
            separator = "&";
            value = values[i];
        }
    }
    return queryString;
}
function getSettingsFromQueryString(queryStringOrParams, paramPrefix) {
    const settings = {};
    if (typeof queryStringOrParams === "string") {
        extractSettingFromQueryString(queryStringOrParams, paramPrefix, settings);
    }
    else if (queryStringOrParams != null) {
        extractSettingsFromQueryParams(queryStringOrParams, paramPrefix, settings);
    }
    return settings;
}
function extractSettingsFromQueryParams(queryParams, paramPrefix, settings) {
    for (const key in queryParams) {
        if (!Object.prototype.hasOwnProperty.call(queryParams, key))
            continue;
        const values = queryParams[key];
        let value, length;
        if (!isArray(values))
            value = values, length = 1;
        else if (values.length)
            value = values[0], length = values.length;
        else
            continue;
        for (let i = 0;;) {
            extractSettingFromQueryParam(key, value, paramPrefix, settings);
            if (++i >= length)
                break;
            value = values[i];
        }
    }
}
function extractSettingFromQueryString(queryString, paramPrefix, settings) {
    if (!queryString
        || queryString.lastIndexOf("?", 0) < 0) {
        return;
    }
    const parts = queryString.substring(1).split("&");
    for (let part of parts) {
        part = part.replace(/\+/g, " ");
        const index = part.indexOf("=");
        const key = decodeURIComponent(index >= 0 ? part.substring(0, index) : part);
        const value = index >= 0 ? decodeURIComponent(part.substring(index + 1)) : "";
        extractSettingFromQueryParam(key, value, paramPrefix, settings);
    }
}
function extractSettingFromQueryParam(key, value, paramPrefix, settings) {
    if (!key
        || key.length <= paramPrefix.length
        || key.lastIndexOf(paramPrefix, 0) < 0) {
        return;
    }
    key = key.substring(paramPrefix.length);
    const interpretValueAsString = key.length > FORCE_STRING_VALUE_SUFFIX.length
        && key.indexOf(FORCE_STRING_VALUE_SUFFIX, key.length - FORCE_STRING_VALUE_SUFFIX.length) >= 0;
    if (interpretValueAsString) {
        key = key.substring(0, key.length - FORCE_STRING_VALUE_SUFFIX.length);
    }
    else {
        value = parseSettingValue(value);
    }
    settings[key] = Setting.fromValue(value);
}
function parseSettingValue(value) {
    switch (value.toLowerCase()) {
        case "false":
            return false;
        case "true":
            return true;
        default:
            const number = parseFloatStrict(value);
            return !isNaN(number) ? number : value;
    }
}
//# sourceMappingURL=FlagOverrides.js.map