import * as ConfigJson from "./ConfigJson.js";
export { ConfigJson };
export class ProjectConfig {
    static equals(projectConfig1, projectConfig2) {
        return projectConfig1.httpETag && projectConfig2.httpETag
            ? projectConfig1.httpETag === projectConfig2.httpETag
            : projectConfig1.configJson === projectConfig2.configJson;
    }
    constructor(configJson, config, timestamp, httpETag) {
        this.configJson = configJson;
        this.config = config;
        this.timestamp = timestamp;
        this.httpETag = httpETag;
    }
    with(timestamp) { return new ProjectConfig(this.configJson, this.config, timestamp, this.httpETag); }
    get isEmpty() { return !this.config; }
    isExpired(expirationMs) {
        return this === ProjectConfig.empty || this.timestamp + expirationMs < ProjectConfig.generateTimestamp();
    }
    static generateTimestamp() {
        return new Date().getTime();
    }
    static serialize(config) {
        var _a, _b;
        return config.timestamp + "\n"
            + ((_a = config.httpETag) !== null && _a !== void 0 ? _a : "") + "\n"
            + ((_b = config.configJson) !== null && _b !== void 0 ? _b : "");
    }
    static deserialize(value) {
        const separatorIndices = Array(2);
        let index = 0;
        for (let i = 0; i < separatorIndices.length; i++) {
            index = value.indexOf("\n", index);
            if (index < 0) {
                throw new Error("Number of values is fewer than expected.");
            }
            separatorIndices[i] = index++;
        }
        let endIndex = separatorIndices[0];
        let slice = value.substring(0, endIndex);
        const fetchTime = parseInt(slice);
        if (isNaN(fetchTime)) {
            throw new Error("Invalid fetch time: " + slice);
        }
        index = endIndex + 1;
        endIndex = separatorIndices[1];
        slice = value.substring(index, endIndex);
        const httpETag = slice.length > 0 ? slice : void 0;
        index = endIndex + 1;
        slice = value.substring(index);
        let config;
        let configJson;
        if (slice.length > 0) {
            config = Config.deserialize(slice);
            configJson = slice;
        }
        return new ProjectConfig(configJson, config, fetchTime, httpETag);
    }
}
ProjectConfig.serializationFormatVersion = "v2";
ProjectConfig.empty = new ProjectConfig(void 0, void 0, 0, void 0);
export class Config {
    static deserialize(configJson) {
        const configJsonParsed = JSON.parse(configJson);
        if (typeof configJsonParsed !== "object" || !configJsonParsed) {
            throw new Error("Invalid config JSON content:" + configJson);
        }
        return new Config(configJsonParsed);
    }
    constructor(json) {
        var _a, _b;
        this.preferences = json.p != null ? new Preferences(json.p) : void 0;
        this.segments = (_b = (_a = json.s) === null || _a === void 0 ? void 0 : _a.map(item => new Segment(item))) !== null && _b !== void 0 ? _b : [];
        this.settings = json.f != null
            ? Object.entries(json.f)
                .reduce((acc, [key, value]) => (acc[key] = new Setting(value, this), acc), {})
            : {};
    }
    get salt() { var _a; return (_a = this.preferences) === null || _a === void 0 ? void 0 : _a.salt; }
}
export class Preferences {
    constructor(json) {
        this.baseUrl = json.u;
        this.redirectMode = json.r;
        this.salt = json.s;
    }
}
export class Segment {
    constructor(json) {
        var _a, _b;
        this.name = json.n;
        this.conditions = (_b = (_a = json.r) === null || _a === void 0 ? void 0 : _a.map(item => new UserCondition(item))) !== null && _b !== void 0 ? _b : [];
    }
}
export class SettingValueContainer {
    constructor(json, hasUnwrappedValue = false) {
        this.value = (!hasUnwrappedValue ? unwrapSettingValue(json.v) : json.v);
        this.variationId = json.i;
    }
}
export class Setting extends SettingValueContainer {
    constructor(json, config) {
        var _a, _b, _c, _d, _e, _f;
        super(json, json.t < 0);
        this.type = json.t;
        this.percentageOptionsAttribute = (_a = json.a) !== null && _a !== void 0 ? _a : "Identifier";
        this.targetingRules = (_c = (_b = json.r) === null || _b === void 0 ? void 0 : _b.map(item => new TargetingRule(item, config))) !== null && _c !== void 0 ? _c : [];
        this.percentageOptions = (_e = (_d = json.p) === null || _d === void 0 ? void 0 : _d.map(item => new PercentageOption(item))) !== null && _e !== void 0 ? _e : [];
        this.configJsonSalt = (_f = config === null || config === void 0 ? void 0 : config.salt) !== null && _f !== void 0 ? _f : "";
    }
    static fromValue(value) {
        return new Setting({
            t: -1,
            v: value,
        });
    }
}
export class TargetingRule {
    constructor(json, config) {
        var _a, _b;
        this.conditions = (_b = (_a = json.c) === null || _a === void 0 ? void 0 : _a.map(item => item.u != null ? new UserCondition(item.u) :
            item.p != null ? new PrerequisiteFlagCondition(item.p) :
                item.s != null ? new SegmentCondition(item.s, config) :
                    void 0)) !== null && _b !== void 0 ? _b : [];
        this.then = json.p != null
            ? json.p.map(item => new PercentageOption(item))
            : new SettingValueContainer(json.s);
    }
}
export class PercentageOption extends SettingValueContainer {
    constructor(json) {
        super(json);
        this.percentage = json.p;
    }
}
export class UserCondition {
    constructor(json) {
        var _a, _b;
        this.type = "UserCondition";
        this.comparisonAttribute = json.a;
        this.comparator = json.c;
        this.comparisonValue = ((_b = (_a = json.s) !== null && _a !== void 0 ? _a : json.d) !== null && _b !== void 0 ? _b : json.l);
    }
}
export class PrerequisiteFlagCondition {
    constructor(json) {
        this.type = "PrerequisiteFlagCondition";
        this.prerequisiteFlagKey = json.f;
        this.comparator = json.c;
        this.comparisonValue = unwrapSettingValue(json.v);
    }
}
export class SegmentCondition {
    constructor(json, config) {
        this.type = "SegmentCondition";
        this.segment = config.segments[json.s];
        this.comparator = json.c;
    }
}
function unwrapSettingValue(json) {
    var _a, _b, _c;
    return ((_c = (_b = (_a = json.b) !== null && _a !== void 0 ? _a : json.s) !== null && _b !== void 0 ? _b : json.i) !== null && _c !== void 0 ? _c : json.d);
}
export function nameOfSettingType(value) {
    return ConfigJson.SettingType[value];
}
//# sourceMappingURL=ProjectConfig.js.map