export class AbortToken {
    constructor() {
        this.callbacks = [];
    }
    get aborted() { return !this.callbacks; }
    abort() {
        if (!this.aborted) {
            const callbacks = this.callbacks;
            this.callbacks = null;
            for (const callback of callbacks) {
                callback();
            }
        }
    }
    registerCallback(callback) {
        if (this.aborted) {
            callback();
            return () => { };
        }
        this.callbacks.push(callback);
        return () => {
            const callbacks = this.callbacks;
            let index;
            if (callbacks && (index = callbacks.indexOf(callback)) >= 0) {
                callbacks.splice(index, 1);
            }
        };
    }
}
export function delay(delayMs, abortToken) {
    let timerId;
    return new Promise(resolve => {
        const unregisterAbortCallback = abortToken === null || abortToken === void 0 ? void 0 : abortToken.registerCallback(() => {
            clearTimeout(timerId);
            resolve(false);
        });
        timerId = setTimeout(() => {
            unregisterAbortCallback === null || unregisterAbortCallback === void 0 ? void 0 : unregisterAbortCallback();
            resolve(true);
        }, delayMs);
    });
}
export function errorToString(err, includeStackTrace = false) {
    return err instanceof Error
        ? includeStackTrace && err.stack ? err.stack : err.toString()
        : err + "";
}
export function throwError(err) {
    throw err;
}
export function isArray(value) {
    return Array.isArray(value);
}
export function isStringArray(value) {
    return isArray(value) && !value.some(item => typeof item !== "string");
}
export function formatStringList(items, maxLength = 0, getOmittedItemsText, separator = ", ") {
    const length = items.length;
    if (!length) {
        return "";
    }
    let appendix = "";
    if (maxLength > 0 && length > maxLength) {
        items = items.slice(0, maxLength);
        if (getOmittedItemsText) {
            appendix = getOmittedItemsText(length - maxLength);
        }
    }
    return "'" + items.join("'" + separator + "'") + "'" + appendix;
}
export function isPromiseLike(obj) {
    return typeof (obj === null || obj === void 0 ? void 0 : obj.then) === "function";
}
export function utf8Encode(text) {
    function codePointAt(text, index) {
        const ch = text.charCodeAt(index);
        if (0xD800 <= ch && ch < 0xDC00) {
            const nextCh = text.charCodeAt(index + 1);
            if (0xDC00 <= nextCh && nextCh <= 0xDFFF) {
                return (ch << 10) + nextCh - 0x35FDC00;
            }
        }
        return ch;
    }
    let utf8text = "", chunkStart = 0;
    const fromCharCode = String.fromCharCode;
    let i;
    for (i = 0; i < text.length; i++) {
        const cp = codePointAt(text, i);
        if (cp <= 0x7F) {
            continue;
        }
        utf8text += text.slice(chunkStart, i);
        if (cp <= 0x7FF) {
            utf8text += fromCharCode(0xC0 | (cp >> 6));
            utf8text += fromCharCode(0x80 | (cp & 0x3F));
        }
        else if (cp <= 0xFFFF) {
            utf8text += fromCharCode(0xE0 | (cp >> 12));
            utf8text += fromCharCode(0x80 | ((cp >> 6) & 0x3F));
            utf8text += fromCharCode(0x80 | (cp & 0x3F));
        }
        else {
            utf8text += fromCharCode(0xF0 | (cp >> 18));
            utf8text += fromCharCode(0x80 | ((cp >> 12) & 0x3F));
            utf8text += fromCharCode(0x80 | ((cp >> 6) & 0x3F));
            utf8text += fromCharCode(0x80 | (cp & 0x3F));
            ++i;
        }
        chunkStart = i + 1;
    }
    return utf8text += text.slice(chunkStart, i);
}
export function parseFloatStrict(value) {
    if (typeof value === "number") {
        return value;
    }
    if (typeof value !== "string" || !value.length || /^\s*$|^\s*0[^\d.e]/.test(value)) {
        return NaN;
    }
    return +value;
}
//# sourceMappingURL=Utils.js.map