import * as http from "http";
import * as https from "https";
import * as tunnel from "tunnel";
import { URL } from "url";
import { FormattableLogMessage } from "../ConfigCatLogger.js";
import { FetchError } from "../ConfigFetcher.js";
export class NodeHttpConfigFetcher {
    constructor(options) {
        this.proxy = options === null || options === void 0 ? void 0 : options.proxy;
    }
    handleResponse(response, resolve, reject) {
        try {
            const { statusCode, statusMessage: reasonPhrase } = response;
            if (statusCode === 200) {
                const eTag = response.headers["etag"];
                const chunks = [];
                response
                    .on("data", chunk => chunks.push(chunk))
                    .on("end", () => {
                    try {
                        resolve({ statusCode, reasonPhrase, eTag, body: Buffer.concat(chunks).toString() });
                    }
                    catch (err) {
                        reject(err);
                    }
                })
                    .on("error", err => reject(new FetchError("failure", err)));
            }
            else {
                response.resume();
                resolve({ statusCode, reasonPhrase });
            }
        }
        catch (err) {
            reject(err);
        }
    }
    fetchLogic(options, lastEtag) {
        return new Promise((resolve, reject) => {
            try {
                options.logger.debug("HttpConfigFetcher.fetchLogic() called.");
                const baseUrl = options.getUrl();
                const isBaseUrlSecure = baseUrl.startsWith("https");
                let agent;
                if (this.proxy) {
                    try {
                        const proxy = new URL(this.proxy);
                        let agentFactory;
                        if (proxy.protocol === "https:") {
                            agentFactory = isBaseUrlSecure ? tunnel.httpsOverHttps : tunnel.httpOverHttps;
                        }
                        else {
                            agentFactory = isBaseUrlSecure ? tunnel.httpsOverHttp : tunnel.httpOverHttp;
                        }
                        agent = agentFactory({
                            proxy: {
                                host: proxy.hostname,
                                port: proxy.port,
                                proxyAuth: (proxy.username && proxy.password) ? `${proxy.username}:${proxy.password}` : null
                            }
                        });
                    }
                    catch (err) {
                        options.logger.log(1, 0, FormattableLogMessage.from("PROXY") `Failed to parse \`options.proxy\`: '${this.proxy}'.`, err);
                    }
                }
                const headers = {
                    "User-Agent": options.clientVersion
                };
                if (lastEtag) {
                    headers["If-None-Match"] = lastEtag;
                }
                const requestOptions = {
                    agent,
                    headers,
                    timeout: options.requestTimeoutMs,
                };
                options.logger.debug(JSON.stringify(requestOptions));
                const request = (isBaseUrlSecure ? https : http).get(baseUrl, requestOptions, response => this.handleResponse(response, resolve, reject))
                    .on("timeout", () => {
                    try {
                        request.destroy();
                    }
                    finally {
                        reject(new FetchError("timeout", options.requestTimeoutMs));
                    }
                })
                    .on("error", err => {
                    reject(new FetchError("failure", err));
                })
                    .end();
            }
            catch (err) {
                reject(err);
            }
        });
    }
}
//# sourceMappingURL=NodeHttpConfigFetcher.js.map